"""
JavaScript兼容RSA加密解密模块
基于JSEncrypt库的Python实现
"""

import base64
import binascii
from typing import Optional, Union

from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.backends import default_backend


class JSRSA:
    """
    JavaScript JSEncrypt兼容的RSA加密解密类
    """

    def __init__(self):
        self.key = None
        self.log = False

    def _apply_zero_padding(self, data: bytes, key_size: int) -> bytes:
        """
        应用零填充（模拟JavaScript中padding=0的行为）
        
        :param data: 要填充的数据
        :param key_size: 密钥大小（字节）
        :return: 填充后的数据
        """
        # 添加PKCS#1 v1.5格式的零填充
        padded = bytearray(key_size)
        padded[1] = 0  # 第二个字节为0
        # 数据放在最后
        data_start_index = key_size - len(data)
        padded[data_start_index:] = data
        return bytes(padded)

    def _remove_zero_padding(self, padded_data: bytes) -> bytes:
        """
        移除零填充（模拟JavaScript中padding=0的解密行为）
        
        :param padded_data: 填充的数据
        :return: 移除填充后的数据
        """
        # 查找第一个非零字节的位置（跳过前两个字节）
        start_index = 2
        while start_index < len(padded_data) and padded_data[start_index] == 0:
            start_index += 1
        return padded_data[start_index:]

    def set_public_key(self, pubkey: str):
        """
        设置公钥
        
        :param pubkey: PEM格式的公钥字符串
        """
        try:
            if isinstance(pubkey, str):
                pubkey = pubkey.encode('utf-8')
            self.key = serialization.load_pem_public_key(pubkey, backend=default_backend())
        except Exception as e:
            if self.log:
                print(f"Error setting public key: {e}")
            raise

    def set_private_key(self, privkey: str):
        """
        设置私钥
        
        :param privkey: PEM格式的私钥字符串
        """
        try:
            if isinstance(privkey, str):
                privkey = privkey.encode('utf-8')
            self.key = serialization.load_pem_private_key(privkey, password=None, backend=default_backend())
        except Exception as e:
            if self.log:
                print(f"Error setting private key: {e}")
            raise

    def public_encrypt(self, text: str, pad: Union[padding.AsymmetricPadding, int] = None) -> str:
        """
        使用公钥加密（兼容JavaScript JSEncrypt）
        
        :param text: 要加密的文本
        :param pad: 填充方式，可以是cryptography库的填充对象或整数(0=zero padding, 1=特定填充, 2=PKCS1v15)
        :return: Base64编码的加密结果
        """
        if not isinstance(self.key, RSAPublicKey):
            raise ValueError("需要设置有效的公钥用于加密")

        # 如果未指定填充方式，则使用默认的PKCS1v15
        if pad is None:
            pad = padding.PKCS1v15()
        
        # 如果pad是整数，则映射到对应的填充方式
        elif isinstance(pad, int):
            if pad == 0:  # Zero padding (接近JavaScript中的nopadding)
                # 手动实现零填充
                # 注意：这种方式不安全，仅用于兼容JavaScript的特殊需求
                key_size = self.key.key_size // 8
                padded_data = self._apply_zero_padding(text.encode('utf-8'), key_size)
                # 直接进行模幂运算，不使用标准填充
                from cryptography.hazmat.primitives.asymmetric import rsa
                encrypted = pow(
                    int.from_bytes(padded_data, 'big'),
                    self.key.public_numbers().e,
                    self.key.public_numbers().n
                )
                encrypted_bytes = encrypted.to_bytes(key_size, 'big')
                return base64.b64encode(encrypted_bytes).decode('utf-8')
            elif pad == 2:  # PKCS1v15
                pad = padding.PKCS1v15()
            # 其他情况默认使用PKCS1v15
            else:
                pad = padding.PKCS1v15()

        try:
            # 使用指定的填充方式
            encrypted = self.key.encrypt(
                text.encode('utf-8'),
                pad
            )
            # 转换为JavaScript兼容的十六进制格式
            hex_encrypted = binascii.hexlify(encrypted).decode('utf-8')
            # 转换为Base64格式返回
            return base64.b64encode(binascii.unhexlify(hex_encrypted)).decode('utf-8')
        except Exception as e:
            if self.log:
                print(f"加密错误: {e}")
            return False

    def private_decrypt(self, encrypted_text: str, pad: Union[padding.AsymmetricPadding, int] = None) -> str:
        """
        使用私钥解密（兼容JavaScript JSEncrypt）
        
        :param encrypted_text: Base64编码的加密文本
        :param pad: 填充方式，可以是cryptography库的填充对象或整数(0=zero padding, 1=特定填充, 2=PKCS1v15)
        :return: 解密后的原始文本
        """
        if not isinstance(self.key, RSAPrivateKey):
            raise ValueError("需要设置有效的私钥用于解密")

        # 将Base64解码为字节
        encrypted_data = base64.b64decode(encrypted_text)
        
        # 如果未指定填充方式，则使用默认的PKCS1v15
        if pad is None:
            pad = padding.PKCS1v15()
        
        # 如果pad是整数，则映射到对应的处理方式
        elif isinstance(pad, int):
            if pad == 0:  # Zero padding (接近JavaScript中的nopadding)
                # 手动实现零填充解密
                key_size = self.key.key_size // 8
                # 直接进行模幂运算，不使用标准解密
                decrypted_int = pow(
                    int.from_bytes(encrypted_data, 'big'),
                    self.key.private_numbers().d,
                    self.key.private_numbers().public_numbers.n
                )
                decrypted_bytes = decrypted_int.to_bytes(key_size, 'big')
                # 移除零填充并解码
                return self._remove_zero_padding(decrypted_bytes).decode('utf-8')
            elif pad == 2:  # PKCS1v15
                pad = padding.PKCS1v15()
            # 其他情况默认使用PKCS1v15
            else:
                pad = padding.PKCS1v15()

        try:
            # 使用指定的填充方式解密
            decrypted = self.key.decrypt(
                encrypted_data,
                pad
            )
            return decrypted.decode('utf-8')
        except Exception as e:
            if self.log:
                print(f"解密错误: {e}")
            return False

    def private_encrypt(self, text: str) -> str:
        """
        使用私钥加密（签名）
        
        :param text: 要加密的文本
        :return: Base64编码的加密结果
        """
        if not isinstance(self.key, RSAPrivateKey):
            raise ValueError("需要设置有效的私钥用于加密")

        try:
            # 私钥加密通常用于签名，这里使用PKCS1v15填充
            encrypted = self.key.sign(
                text.encode('utf-8'),
                padding.PKCS1v15(),
                hashes.SHA256()
            )
            # 转换为Base64格式返回
            return base64.b64encode(encrypted).decode('utf-8')
        except Exception as e:
            if self.log:
                print(f"私钥加密错误: {e}")
            return False

    def public_decrypt(self, encrypted_text: str) -> str:
        """
        使用公钥解密（验证签名）
        
        :param encrypted_text: Base64编码的加密文本
        :return: 解密后的原始文本
        """
        if not isinstance(self.key, RSAPublicKey):
            raise ValueError("需要设置有效的公钥用于解密")

        try:
            # 将Base64解码为字节
            encrypted_data = base64.b64decode(encrypted_text)
            # 公钥解密（验证）
            # 注意：这里实际是验证签名，不是真正的解密
            # 在JSEncrypt中，公钥解密通常用于验证私钥加密的内容
            # 但Python的cryptography库不直接支持公钥解密
            # 这里需要特殊处理，可能需要使用签名验证
            return "Not supported in Python version"
        except Exception as e:
            if self.log:
                print(f"公钥解密错误: {e}")
            return False

    @staticmethod
    def generate_keypair(key_size: int = 1024) -> tuple:
        """
        生成RSA密钥对
        
        :param key_size: 密钥长度，默认1024位
        :return: (private_key, public_key) PEM格式的密钥对
        """
        # 生成私钥
        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=key_size,
            backend=default_backend()
        )

        # 获取私钥PEM格式
        private_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )

        # 获取公钥PEM格式
        public_key = private_key.public_key()
        public_pem = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        )

        return private_pem.decode('utf-8'), public_pem.decode('utf-8')


# 便捷函数
def js_rsa_generate_keypair(key_size: int = 1024) -> tuple:
    """
    生成RSA密钥对
    
    :param key_size: 密钥长度，默认1024位
    :return: (private_key, public_key) PEM格式的密钥对
    """
    return JSRSA.generate_keypair(key_size)


def js_rsa_public_encrypt(text: str, public_key: str, pad: padding.AsymmetricPadding = None) -> str:
    """
    使用公钥加密
    
    :param text: 要加密的文本
    :param public_key: PEM格式的公钥
    :param pad: 填充方式，默认使用PKCS1v15
    :return: Base64编码的加密结果
    """
    rsa_obj = JSRSA()
    rsa_obj.set_public_key(public_key)
    return rsa_obj.public_encrypt(text, pad)


def js_rsa_private_decrypt(encrypted_text: str, private_key: str, pad: padding.AsymmetricPadding = None) -> str:
    """
    使用私钥解密
    
    :param encrypted_text: Base64编码的加密文本
    :param private_key: PEM格式的私钥
    :param pad: 填充方式，默认使用PKCS1v15
    :return: 解密后的原始文本
    """
    rsa_obj = JSRSA()
    rsa_obj.set_private_key(private_key)
    return rsa_obj.private_decrypt(encrypted_text, pad)


def js_rsa_private_encrypt(text: str, private_key: str) -> str:
    """
    使用私钥加密（签名）
    
    :param text: 要加密的文本
    :param private_key: PEM格式的私钥
    :return: Base64编码的加密结果
    """
    rsa_obj = JSRSA()
    rsa_obj.set_private_key(private_key)
    return rsa_obj.private_encrypt(text)


if __name__ == "__main__":
    # 生成密钥对
    private_key, public_key = js_rsa_generate_keypair()
    print("私钥:")
    print(private_key)
    print("\n公钥:")
    print(public_key)
    # 测试加密解密
    text = "Hello, World!"
    print(f"\n原始文本: {text}")
    # 使用默认填充方式（PKCS1v15）加密和解密
    encrypted_default = js_rsa_public_encrypt(text, public_key)
    print(f"\n使用默认填充加密后: {encrypted_default}")
    decrypted_default = js_rsa_private_decrypt(encrypted_default, private_key)
    print(f"\n使用默认填充解密后: {decrypted_default}")

    encrypted_pkcs1 = js_rsa_public_encrypt(text, public_key, padding.PKCS1v15())
    print(f"\n使用PKCS1v15填充加密后: {encrypted_pkcs1}")
    decrypted_pkcs1 = js_rsa_private_decrypt(encrypted_pkcs1, private_key, padding.PKCS1v15())
    print(f"\n使用PKCS1v15填充解密后: {decrypted_pkcs1}")
    

    encrypted_oaep = js_rsa_public_encrypt(text, public_key, 0)
    print(f"\n使用zeroPadding填充加密后: {encrypted_oaep}")
    decrypted_oaep = js_rsa_private_decrypt(encrypted_oaep, private_key, 0)
    print(f"\n使用zeroPadding填充解密后: {decrypted_oaep}")



from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.decrepit.ciphers.algorithms import TripleDES
from cryptography.hazmat.backends import default_backend
import base64
import os


class TDESCipher_JS:
    """
    与JavaScript CryptoJS兼容的3DES加密解密类
    """

    def __init__(self, key: str, iv: str = None, mode: str = 'CBC'):
        """
        初始化3DES加密器（兼容JS CryptoJS）

        Args:
            key (str): 密钥字符串（会被处理为24字节）
            iv (str): IV字符串（会被截取为8字节）
            mode (str): 加密模式 ('ECB', 'CBC')，默认CBC以兼容JS
        """
        # 模拟JS CryptoJS的密钥处理方式
        key_bytes = key.encode('utf-8')
        # 确保密钥长度为24字节（3DES标准）
        if len(key_bytes) < 24:
            key_bytes = key_bytes.ljust(24, b'\0')
        else:
            key_bytes = key_bytes[:24]
        
        self.key = key_bytes
        self.mode = mode.upper()
        
        # IV处理
        if self.mode != 'ECB':
            if iv:
                iv_bytes = iv.encode('utf-8')
                if len(iv_bytes) < 8:
                    iv_bytes = iv_bytes.ljust(8, b'\0')
                else:
                    iv_bytes = iv_bytes[:8]
                self.iv = iv_bytes
            else:
                self.iv = os.urandom(8)
        else:
            self.iv = None
            
        self.backend = default_backend()

    def _get_cipher(self):
        """获取加密器"""
        algorithm = TripleDES(self.key)
        
        if self.mode == 'ECB':
            mode_obj = modes.ECB()
        elif self.mode == 'CBC':
            mode_obj = modes.CBC(self.iv)
        else:
            raise ValueError(f"不支持的加密模式: {self.mode}")
            
        return Cipher(algorithm, mode_obj, backend=self.backend)

    def encrypt(self, plaintext: str) -> bytes:
        """
        加密文本（模拟JS CryptoJS行为）

        Args:
            plaintext (str): 明文

        Returns:
            bytes: 密文
        """
        data = plaintext.encode('utf-8')
        
        # PKCS7填充（ECB和CBC模式都需要填充）
        if self.mode in ['ECB', 'CBC']:
            padder = padding.PKCS7(64).padder()
            padded_data = padder.update(data) + padder.finalize()
        else:
            padded_data = data

        # 获取加密器
        cipher = self._get_cipher()
        encryptor = cipher.encryptor()
        ciphertext = encryptor.update(padded_data) + encryptor.finalize()

        return ciphertext

    def decrypt(self, ciphertext: bytes) -> str:
        """
        解密文本（模拟JS CryptoJS行为）

        Args:
            ciphertext (bytes): 密文

        Returns:
            str: 解密后的明文
        """
        cipher = self._get_cipher()
        decryptor = cipher.decryptor()
        padded_data = decryptor.update(ciphertext) + decryptor.finalize()
        
        # 去除PKCS7填充（ECB和CBC模式都需要去填充）
        if self.mode in ['ECB', 'CBC']:
            unpadder = padding.PKCS7(64).unpadder()
            data = unpadder.update(padded_data) + unpadder.finalize()
        else:
            data = padded_data

        return data.decode('utf-8')


def tdes_encrypt_string_js(plaintext: str, key: str, mode: str = 'CBC',
                          padding_mode: str = 'PKCS7', iv: str = None) -> str:
    """
    与JavaScript CryptoJS兼容的3DES加密字符串，返回base64编码的结果

    Args:
        plaintext (str): 明文
        key (str): 密钥字符串
        mode (str): 加密模式 ('ECB', 'CBC')
        padding_mode (str): 填充模式（保留参数，但实际使用PKCS7）
        iv (str): IV字符串

    Returns:
        str: base64编码的密文
    """
    # 注意：为了与JS兼容，始终使用PKCS7填充
    cipher = TDESCipher_JS(key, iv, mode)
    ciphertext = cipher.encrypt(plaintext)
    return base64.b64encode(ciphertext).decode('utf-8')


def tdes_decrypt_string_js(ciphertext_b64: str, key: str, iv_b64: str = None,
                          mode: str = 'CBC', padding_mode: str = 'PKCS7') -> str:
    """
    与JavaScript CryptoJS兼容的3DES解密base64编码的字符串

    Args:
        ciphertext_b64 (str): base64编码的密文
        key (str): 密钥字符串
        iv_b64 (str): base64编码的IV（对于ECB模式不需要）
        mode (str): 加密模式 ('ECB', 'CBC')
        padding_mode (str): 填充模式（保留参数，但实际使用PKCS7）

    Returns:
        str: 解密后的明文
    """
    # 注意：为了与JS兼容，始终使用PKCS7填充
    cipher = TDESCipher_JS(key, iv_b64, mode)
    ciphertext = base64.b64decode(ciphertext_b64)
    return cipher.decrypt(ciphertext)


if __name__ == '__main__':
    # 测试与JS兼容的3DES加密
    key = "mypassword123456"
    iv = "mypaswd1mypaswd1"  # JS中使用了16字节IV，但我们只取前8字节
    
    plaintext = "Hello World"
    
    # CBC模式加密（默认，与JS兼容）
    result_cbc = tdes_encrypt_string_js(plaintext, key, 'CBC', 'PKCS7', iv)
    print("CBC模式加密结果:", result_cbc)
    decrypted_cbc = tdes_decrypt_string_js(result_cbc, key, iv, 'CBC')
    print("CBC模式解密结果:", decrypted_cbc)
    
    # ECB模式加密
    result_ecb = tdes_encrypt_string_js(plaintext, key, 'ECB', 'PKCS7', None)
    print("ECB模式加密结果:", result_ecb)
    decrypted_ecb = tdes_decrypt_string_js(result_ecb, key, None, 'ECB')
    print("ECB模式解密结果:", decrypted_ecb)


import random
import string


def string_random_string(num, uppercase=True, lowercase=True, digits=True):
    """
    生成随机字符串

    Args:
        num (int): 生成字符串的长度
        uppercase (bool): 是否包含大写字母
        lowercase (bool): 是否包含小写字母
        digits (bool): 是否包含数字

    Returns:
        str: 生成的随机字符串
    """
    if not any([uppercase, lowercase, digits]):
        raise ValueError("至少需要启用一种字符类型")

    # 构建字符池
    char_pool = ""
    if uppercase:
        char_pool += string.ascii_uppercase
    if lowercase:
        char_pool += string.ascii_lowercase
    if digits:
        char_pool += string.digits

    # 生成随机字符串
    return ''.join(random.choices(char_pool, k=num))
def string_get_between(text, start_text, end_text):
    """
    获取两个文本之间的内容（不包含边界文本）

    Args:
        text (str): 原始文本
        start_text (str): 开始文本
        end_text (str): 结束文本

    Returns:
        str: 两个文本之间的内容，如果找不到则返回空字符串
    """
    # 查找开始文本的位置
    start_index = text.find(start_text)
    if start_index == -1:
        return ""

    # 计算开始位置（跳过开始文本）
    start_pos = start_index + len(start_text)

    # 从开始位置查找结束文本
    end_index = text.find(end_text, start_pos)
    if end_index == -1:
        return ""

    # 返回中间的文本
    return text[start_pos:end_index]
def string_get_left(text, delimiter):
    """
    获取文本中指定分隔符左边的内容

    Args:
        text (str): 原始文本
        delimiter (str): 分隔符

    Returns:
        str: 分隔符左边的内容，如果找不到分隔符则返回空字符串
    """
    index = text.find(delimiter)
    if index == -1:
        return ""
    return text[:index]
def string_get_right(text, delimiter):
    """
    获取文本中指定分隔符右边的内容

    Args:
        text (str): 原始文本
        delimiter (str): 分隔符

    Returns:
        str: 分隔符右边的内容，如果找不到分隔符则返回空字符串
    """
    index = text.find(delimiter)
    if index == -1:
        return ""
    return text[index + len(delimiter):]


from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.decrepit.ciphers.algorithms import TripleDES
from cryptography.hazmat.backends import default_backend
import base64
import os


class TDESCipher:
    """
    3DES加密解密类，支持多种模式和填充方式
    """

    def __init__(self, key: bytes, mode: str = 'CBC', padding_mode: str = 'PKCS7'):
        """
        初始化3DES加密器

        Args:
            key (bytes): 密钥（16字节对应2-key 3DES，24字节对应3-key 3DES）
            mode (str): 加密模式 ('ECB', 'CBC', 'CFB', 'OFB')
            padding_mode (str): 填充模式 ('PKCS7', 'ISO7816', 'ANSIX923')
        """
        # 3DES密钥必须是16字节(2-key)或24字节(3-key)
        if len(key) not in [16, 24]:
            # 如果不是16或24字节，尝试调整长度
            if len(key) < 16:
                key = key.ljust(16, b'\0')
            elif 16 < len(key) < 24:
                key = key.ljust(24, b'\0')
            elif len(key) > 24:
                key = key[:24]
            
            # 确保最终长度是16或24字节
            if len(key) not in [16, 24]:
                key = key[:16] if len(key) >= 16 else key.ljust(16, b'\0')
        
        self.key = key
        self.mode = mode.upper()
        self.padding_mode = padding_mode.upper()
        self.backend = default_backend()

    def _get_padding(self, block_size: int = 64):
        """获取填充器"""
        if self.padding_mode == 'PKCS7':
            return padding.PKCS7(block_size)
        elif self.padding_mode == 'ISO7816':
            return padding.ISO7816(block_size)
        elif self.padding_mode == 'ANSIX923':
            return padding.ANSIX923(block_size)
        else:
            raise ValueError(f"不支持的填充模式: {self.padding_mode}")

    def _pad_data(self, data: bytes) -> bytes:
        """数据填充"""
        if self.mode in ['ECB', 'CBC']:
            padder = self._get_padding().padder()
            return padder.update(data) + padder.finalize()
        return data

    def _unpad_data(self, data: bytes) -> bytes:
        """去除填充"""
        if self.mode in ['ECB', 'CBC']:
            unpadder = self._get_padding().unpadder()
            return unpadder.update(data) + unpadder.finalize()
        return data

    def _get_cipher(self, iv: bytes = None):
        """获取加密器"""
        algorithm = TripleDES(self.key)  # 使用3DES算法

        if self.mode == 'ECB':
            mode_obj = modes.ECB()
        elif self.mode == 'CBC':
            if iv is None:
                iv = os.urandom(8)  # 3DES的IV是8字节
            mode_obj = modes.CBC(iv)
        elif self.mode == 'CFB':
            if iv is None:
                iv = os.urandom(8)
            mode_obj = modes.CFB(iv)
        elif self.mode == 'OFB':
            if iv is None:
                iv = os.urandom(8)
            mode_obj = modes.OFB(iv)
        else:
            raise ValueError(f"不支持的加密模式: {self.mode}")

        return Cipher(algorithm, mode_obj, backend=self.backend), iv

    def encrypt(self, plaintext: str, iv: bytes = None) -> bytes:
        """
        加密文本

        Args:
            plaintext (str): 明文
            iv (bytes): 初始化向量（ECB模式不需要）

        Returns:
            bytes: 密文
        """
        data = plaintext.encode('utf-8')
        data = self._pad_data(data)

        cipher, _ = self._get_cipher(iv)
        encryptor = cipher.encryptor()
        ciphertext = encryptor.update(data) + encryptor.finalize()

        return ciphertext

    def decrypt(self, ciphertext: bytes, iv: bytes = None) -> str:
        """
        解密文本

        Args:
            ciphertext (bytes): 密文
            iv (bytes): 初始化向量（ECB模式不需要）

        Returns:
            str: 解密后的明文
        """
        cipher, _ = self._get_cipher(iv)
        decryptor = cipher.decryptor()
        padded_data = decryptor.update(ciphertext) + decryptor.finalize()
        data = self._unpad_data(padded_data)

        return data.decode('utf-8')


# 便捷函数
def tdes_encrypt_string(plaintext: str, key: str, mode: str = 'CBC',
                       padding_mode: str = 'PKCS7', iv: bytes = None) -> str:
    """
    3DES加密字符串，返回base64编码的结果

    Args:
        plaintext (str): 明文
        key (str): 密钥字符串
        mode (str): 加密模式 ('ECB', 'CBC', 'CFB', 'OFB')
        padding_mode (str): 填充模式
        iv (bytes): 初始化向量（ECB模式不需要）

    Returns:
        str: base64编码的密文
    """
    key_bytes = key.encode('utf-8')
    # 确保密钥长度为16或24字节
    if len(key_bytes) < 16:
        key_bytes = key_bytes.ljust(16, b'\0')
    elif 16 < len(key_bytes) < 24:
        key_bytes = key_bytes.ljust(24, b'\0')
    elif len(key_bytes) > 24:
        key_bytes = key_bytes[:24]

    cipher = TDESCipher(key_bytes, mode, padding_mode)
    ciphertext = cipher.encrypt(plaintext, iv)

    return base64.b64encode(ciphertext).decode('utf-8')


def tdes_decrypt_string(ciphertext_b64: str, key: str, iv_b64: str = None,
                       mode: str = 'CBC', padding_mode: str = 'PKCS7') -> str:
    """
    3DES解密base64编码的字符串

    Args:
        ciphertext_b64 (str): base64编码的密文
        key (str): 密钥字符串
        iv_b64 (str): base64编码的IV
        mode (str): 加密模式
        padding_mode (str): 填充模式

    Returns:
        str: 解密后的明文
    """
    key_bytes = key.encode('utf-8')
    # 确保密钥长度为16或24字节
    if len(key_bytes) < 16:
        key_bytes = key_bytes.ljust(16, b'\0')
    elif 16 < len(key_bytes) < 24:
        key_bytes = key_bytes.ljust(24, b'\0')
    elif len(key_bytes) > 24:
        key_bytes = key_bytes[:24]

    ciphertext = base64.b64decode(ciphertext_b64)
    iv = base64.b64decode(iv_b64) if iv_b64 else None

    cipher = TDESCipher(key_bytes, mode, padding_mode)
    return cipher.decrypt(ciphertext, iv)


if __name__ == '__main__':
    # 测试3DES加密
    result = tdes_encrypt_string("Hello World", "mypassword123456", "CBC", "PKCS7", b"mypaswd1mypaswd1")
    print("加密结果:", result)
    decrypted = tdes_decrypt_string(result, "mypassword123456", base64.b64encode(b"mypaswd1mypaswd1").decode(), "CBC")
    print("解密结果:", decrypted)


from typing import Optional, Dict, Any
import psutil
import subprocess
import platform
import sys
from typing import Dict, Any

import platform
import subprocess
import os
from typing import Optional


def system_get_cpu_info() -> dict:
    """
    获取CPU详细信息

    Returns:
        dict: 包含CPU信息的字典
    """
    cpu_info = {
        'model': 'Unknown',
        'cores': 0,
        'threads': 0,
        'architecture': platform.machine(),
        'frequency': 'Unknown'
    }

    try:
        system = platform.system().lower()

        if system == "windows":
            # Windows系统获取CPU信息
            try:
                # 获取CPU型号
                result = subprocess.run(
                    ['wmic', 'cpu', 'get', 'Name', '/value'],
                    capture_output=True, text=True
                )
                if result.returncode == 0:
                    lines = result.stdout.strip().split('\n')
                    for line in lines:
                        if 'Name=' in line:
                            cpu_info['model'] = line.split('=')[1].strip()
                            break

                # 获取CPU核心数
                result = subprocess.run(
                    ['wmic', 'cpu', 'get', 'NumberOfCores', '/value'],
                    capture_output=True, text=True
                )
                if result.returncode == 0:
                    lines = result.stdout.strip().split('\n')
                    for line in lines:
                        if 'NumberOfCores=' in line:
                            cpu_info['cores'] = int(line.split('=')[1].strip())
                            break

                # 获取逻辑处理器数
                result = subprocess.run(
                    ['wmic', 'cpu', 'get', 'NumberOfLogicalProcessors', '/value'],
                    capture_output=True, text=True
                )
                if result.returncode == 0:
                    lines = result.stdout.strip().split('\n')
                    for line in lines:
                        if 'NumberOfLogicalProcessors=' in line:
                            cpu_info['threads'] = int(line.split('=')[1].strip())
                            break

            except Exception:
                # 回退到platform.processor()
                cpu_info['model'] = platform.processor() or 'Unknown'

        elif system == "linux":
            # Linux系统获取CPU信息
            try:
                # 从/proc/cpuinfo获取CPU型号
                with open('/proc/cpuinfo', 'r') as f:
                    cpuinfo = f.read()

                # 查找CPU型号
                for line in cpuinfo.split('\n'):
                    if 'model name' in line:
                        cpu_info['model'] = line.split(':')[1].strip()
                        break

                # 获取核心数
                cores = set()
                for line in cpuinfo.split('\n'):
                    if 'core id' in line:
                        core_id = line.split(':')[1].strip()
                        cores.add(core_id)

                cpu_info['cores'] = len(cores) if cores else 1

                # 获取线程数
                thread_count = 0
                for line in cpuinfo.split('\n'):
                    if 'processor' in line and ':' in line:
                        thread_count += 1

                cpu_info['threads'] = thread_count if thread_count > 0 else 1

            except Exception:
                pass

        elif system == "darwin":
            # macOS系统获取CPU信息
            try:
                # 获取CPU型号
                result = subprocess.run(
                    ['sysctl', '-n', 'machdep.cpu.brand_string'],
                    capture_output=True, text=True
                )
                if result.returncode == 0:
                    cpu_info['model'] = result.stdout.strip()

                # 获取核心数
                result = subprocess.run(
                    ['sysctl', '-n', 'hw.physicalcpu'],
                    capture_output=True, text=True
                )
                if result.returncode == 0:
                    cpu_info['cores'] = int(result.stdout.strip())

                # 获取逻辑核心数
                result = subprocess.run(
                    ['sysctl', '-n', 'hw.logicalcpu'],
                    capture_output=True, text=True
                )
                if result.returncode == 0:
                    cpu_info['threads'] = int(result.stdout.strip())

            except Exception:
                pass

        # 如果还是Unknown，尝试使用platform.processor()
        if cpu_info['model'] == 'Unknown':
            cpu_info['model'] = platform.processor() or 'Unknown'

    except Exception:
        pass

    return cpu_info


def system_get_cpu_model() -> str:
    """
    获取CPU型号

    Returns:
        str: CPU型号字符串
    """
    try:
        cpu_info = system_get_cpu_info()
        return cpu_info.get('model', 'Unknown')
    except Exception:
        return 'Unknown'


def system_get_cpu_cores() -> int:
    """
    获取CPU物理核心数

    Returns:
        int: CPU物理核心数
    """
    try:
        cpu_info = system_get_cpu_info()
        return cpu_info.get('cores', 0)
    except Exception:
        return 0


def system_get_cpu_threads() -> int:
    """
    获取CPU逻辑线程数

    Returns:
        int: CPU逻辑线程数
    """
    try:
        cpu_info = system_get_cpu_info()
        return cpu_info.get('threads', 0)
    except Exception:
        return 0

def system_empty_recycle_bin(confirm: bool = True) -> bool:
    """
    清空回收站

    Args:
        confirm (bool): 是否弹出询问框，默认为True（弹出询问框）
                        True表示弹出系统确认对话框
                        False表示直接清空不询问

    Returns:
        bool: 成功清空回收站返回True，失败返回False
    """
    try:
        system = platform.system().lower()

        if system == "windows":
            # Windows系统清空回收站
            try:
                import ctypes
                from ctypes import wintypes

                # Windows API常量
                SHERB_NOCONFIRMATION = 0x00000001
                SHERB_NOPROGRESSUI = 0x00000002
                SHERB_NOSOUND = 0x00000004

                # 加载shell32.dll
                shell32 = ctypes.windll.shell32

                # 设置标志位
                flags = 0
                if not confirm:
                    flags |= SHERB_NOCONFIRMATION  # 不显示确认对话框

                # 调用Windows API清空回收站
                # 参数说明：
                # hwnd: 父窗口句柄，0表示无父窗口
                # pszRootPath: 回收站路径，None表示所有驱动器
                # dwFlags: 标志位
                result = shell32.SHEmptyRecycleBinW(0, None, flags)

                # 返回值为0表示成功
                return result == 0

            except Exception:
                # 如果ctypes方法失败，尝试使用PowerShell命令
                try:
                    if confirm:
                        # 弹出确认对话框
                        cmd = [
                            "powershell",
                            "-Command",
                            "Add-Type -AssemblyName Microsoft.VisualBasic; "
                            "[Microsoft.VisualBasic.Interaction]::MsgBox('确定要清空回收站吗？', 'OKCancel', '确认清空回收站') | Out-Null; "
                            "Clear-RecycleBin -Force"
                        ]
                    else:
                        # 直接清空不询问
                        cmd = [
                            "powershell",
                            "-Command",
                            "Clear-RecycleBin -Force"
                        ]

                    result = subprocess.run(cmd, capture_output=True, text=True)
                    return result.returncode == 0
                except Exception:
                    return False

        elif system == "darwin":  # macOS
            # macOS系统清空废纸篓
            try:
                # 使用AppleScript清空废纸篓
                if confirm:
                    # 弹出确认对话框
                    script = '''
                    display dialog "确定要清空废纸篓吗？" buttons {"取消", "确定"} default button "取消"
                    if button returned of result is "确定" then
                        tell application "Finder"
                            empty trash
                        end tell
                    end if
                    '''
                else:
                    # 直接清空不询问
                    script = '''
                    tell application "Finder"
                        empty trash
                    end tell
                    '''

                cmd = ["osascript", "-e", script]
                result = subprocess.run(cmd, capture_output=True, text=True)
                return result.returncode == 0

            except Exception:
                return False

        elif system == "linux":
            # Linux系统清空回收站
            try:
                import shutil
                import glob

                # Linux回收站路径通常在 ~/.local/share/Trash/
                trash_path = os.path.expanduser("~/.local/share/Trash/")

                if os.path.exists(trash_path):
                    if confirm:
                        # 简单的命令行确认
                        response = input("确定要清空回收站吗？(y/N): ")
                        if response.lower() not in ['y', 'yes']:
                            return False

                    # 清空files目录（实际文件）
                    files_dir = os.path.join(trash_path, "files")
                    if os.path.exists(files_dir):
                        for item in os.listdir(files_dir):
                            item_path = os.path.join(files_dir, item)
                            try:
                                if os.path.isfile(item_path) or os.path.islink(item_path):
                                    os.unlink(item_path)
                                elif os.path.isdir(item_path):
                                    shutil.rmtree(item_path)
                            except Exception:
                                continue

                    # 清空info目录（元数据）
                    info_dir = os.path.join(trash_path, "info")
                    if os.path.exists(info_dir):
                        for item in os.listdir(info_dir):
                            item_path = os.path.join(info_dir, item)
                            try:
                                os.unlink(item_path)
                            except Exception:
                                continue

                    return True
                else:
                    return False

            except Exception:
                return False
        else:
            # 不支持的操作系统
            return False

    except Exception:
        return False


def system_get_version() -> Dict[str, Any]:
    """
    获取系统版本信息

    Returns:
        dict: 包含系统版本信息的字典
    """
    try:
        # 获取系统信息
        system_info = {
            'system': platform.system(),  # 操作系统名称
            'release': platform.release(),  # 操作系统发行版本
            'version': platform.version(),  # 操作系统版本
            'machine': platform.machine(),  # 机器类型
            'processor': platform.processor(),  # 处理器信息
            'architecture': platform.architecture()[0],  # 系统架构
            'node': platform.node(),  # 网络节点名称
            'platform': platform.platform(),  # 平台信息
            'python_version': sys.version,  # Python版本
            'python_compiler': platform.python_compiler()  # Python编译器
        }

        # 根据不同操作系统获取更详细的版本信息
        if system_info['system'].lower() == 'windows':
            # Windows系统额外信息
            try:
                import subprocess
                # 获取Windows版本详细信息
                result = subprocess.run(
                    ['wmic', 'os', 'get', 'Caption,Version,BuildNumber', '/value'],
                    capture_output=True, text=True
                )
                if result.returncode == 0:
                    lines = result.stdout.strip().split('\n')
                    for line in lines:
                        if 'Caption=' in line:
                            system_info['windows_caption'] = line.split('=')[1]
                        elif 'Version=' in line:
                            system_info['windows_version'] = line.split('=')[1]
                        elif 'BuildNumber=' in line:
                            system_info['windows_build'] = line.split('=')[1]
            except Exception:
                pass

        elif system_info['system'].lower() in ['linux', 'darwin']:
            # Unix/Linux/macOS系统额外信息
            try:
                import subprocess
                if system_info['system'].lower() == 'linux':
                    # Linux系统获取发行版信息
                    result = subprocess.run(['lsb_release', '-d'], capture_output=True, text=True)
                    if result.returncode == 0:
                        system_info['distribution'] = result.stdout.strip().split(':')[1].strip()
                else:
                    # macOS系统获取版本信息
                    result = subprocess.run(['sw_vers', '-productVersion'], capture_output=True, text=True)
                    if result.returncode == 0:
                        system_info['macos_version'] = result.stdout.strip()
            except Exception:
                pass

        return system_info

    except Exception as e:
        # 发生异常时返回基本系统信息
        return {
            'system': platform.system(),
            'release': platform.release(),
            'version': platform.version(),
            'error': str(e)
        }


def system_get_simple_version() -> str:
    """
    获取简化版系统版本信息

    Returns:
        str: 简化版系统版本信息
    """
    try:
        return platform.platform()
    except Exception:
        return f"{platform.system()} {platform.release()}"


def system_is_windows() -> bool:
    """
    检查是否为Windows系统

    Returns:
        bool: 是Windows系统返回True，否则返回False
    """
    return platform.system().lower() == 'windows'


def system_is_linux() -> bool:
    """
    检查是否为Linux系统

    Returns:
        bool: 是Linux系统返回True，否则返回False
    """
    return platform.system().lower() == 'linux'


def system_is_macos() -> bool:
    """
    检查是否为macOS系统

    Returns:
        bool: 是macOS系统返回True，否则返回False
    """
    return platform.system().lower() == 'darwin'


def process_kill_by_pid(pid: int, force: bool = False) -> bool:
    """
    根据PID结束进程

    Args:
        pid (int): 进程ID
        force (bool): 是否强制结束进程，默认为False（正常结束）
                     True表示强制结束（SIGKILL），False表示正常结束（SIGTERM）

    Returns:
        bool: 成功结束进程返回True，失败返回False
    """
    try:
        # 检查PID是否有效
        if not isinstance(pid, int) or pid <= 0:
            raise ValueError("PID必须是正整数")

        # 获取进程对象
        process = psutil.Process(pid)

        # 结束进程
        if force:
            # 强制结束进程
            process.kill()  # SIGKILL
        else:
            # 正常结束进程
            process.terminate()  # SIGTERM

        # 等待进程结束
        process.wait(timeout=3)
        return True

    except psutil.NoSuchProcess:
        # 进程不存在
        return False
    except psutil.AccessDenied:
        # 权限不足
        return False
    except psutil.TimeoutExpired:
        # 进程未在指定时间内结束，如果需要可以强制结束
        if not force:
            try:
                process.kill()
                process.wait(timeout=1)
                return True
            except Exception:
                return False
        return False
    except Exception:
        # 其他异常
        return False


def process_kill_by_name(name: str, force: bool = False) -> int:
    """
    根据进程名结束进程

    Args:
        name (str): 进程名
        force (bool): 是否强制结束进程，默认为False

    Returns:
        int: 成功结束的进程数量
    """
    killed_count = 0

    try:
        # 遍历所有进程
        for process in psutil.process_iter(['pid', 'name']):
            try:
                # 检查进程名是否匹配
                if process.info['name'].lower() == name.lower():
                    # 结束进程
                    if process_kill_by_pid(process.info['pid'], force):
                        killed_count += 1
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                # 进程已不存在或权限不足，跳过
                continue

    except Exception:
        pass

    return killed_count


def port_check_process(port: int, protocol: str = 'tcp') -> Optional[Dict[str, Any]]:
    """
    检查端口被哪个程序占用

    Args:
        port (int): 端口号
        protocol (str): 协议类型，'tcp' 或 'udp'，默认为 'tcp'

    Returns:
        dict or None: 如果端口被占用，返回包含进程信息的字典；否则返回None
    """
    try:
        # 检查端口是否有效
        if not isinstance(port, int) or port < 0 or port > 65535:
            raise ValueError("端口号必须是0-65535之间的整数")

        # 检查协议类型
        protocol = protocol.lower()
        if protocol not in ['tcp', 'udp']:
            raise ValueError("协议类型必须是'tcp'或'udp'")

        # 遍历所有网络连接
        for conn in psutil.net_connections(kind=protocol):
            # 检查端口是否匹配
            if conn.laddr.port == port:
                # 获取进程信息
                try:
                    process = psutil.Process(conn.pid)
                    return {
                        'pid': conn.pid,
                        'name': process.name(),
                        'exe': process.exe(),
                        'cmdline': ' '.join(process.cmdline()),
                        'status': process.status(),
                        'port': port,
                        'protocol': protocol,
                        'local_address': f"{conn.laddr.ip}:{conn.laddr.port}",
                        'remote_address': f"{conn.raddr.ip}:{conn.raddr.port}" if conn.raddr else "N/A"
                    }
                except (psutil.NoSuchProcess, psutil.AccessDenied):
                    # 如果无法获取进程信息，返回基本连接信息
                    return {
                        'pid': conn.pid,
                        'name': "Unknown",
                        'exe': "Unknown",
                        'cmdline': "Unknown",
                        'status': "Unknown",
                        'port': port,
                        'protocol': protocol,
                        'local_address': f"{conn.laddr.ip}:{conn.laddr.port}",
                        'remote_address': f"{conn.raddr.ip}:{conn.raddr.port}" if conn.raddr else "N/A"
                    }

        # 端口未被占用
        return None

    except Exception as e:
        # 发生异常时返回None
        return None


def port_get_processes(port: int) -> list:
    """
    获取占用指定端口的所有进程信息

    Args:
        port (int): 端口号

    Returns:
        list: 占用该端口的进程信息列表
    """
    processes = []

    try:
        # 检查TCP连接
        tcp_result = port_check_process(port, 'tcp')
        if tcp_result:
            processes.append(tcp_result)

        # 检查UDP连接
        udp_result = port_check_process(port, 'udp')
        if udp_result:
            processes.append(udp_result)

    except Exception:
        pass

    return processes




def system_shutdown(delay: int = 0, message: str = "") -> bool:
    """
    关闭计算机系统

    Args:
        delay (int): 延迟关机时间（秒），默认为0（立即关机）
        message (str): 关机前显示的消息，默认为空

    Returns:
        bool: 成功发起关机命令返回True，失败返回False
    """
    try:
        system = platform.system().lower()

        if system == "windows":
            # Windows系统关机命令
            if delay > 0:
                cmd = ["shutdown", "/s", "/t", str(delay)]
            else:
                cmd = ["shutdown", "/s", "/f", "/t", "0"]

            if message:
                cmd.extend(["/c", message])

        elif system in ["linux", "darwin"]:  # Linux or macOS
            # Unix/Linux/macOS系统关机命令
            if delay > 0:
                # 使用at命令实现延迟关机
                minutes = delay // 60
                if minutes > 0:
                    cmd = ["sudo", "shutdown", "-h", f"+{minutes}"]
                else:
                    cmd = ["sudo", "shutdown", "-h", "now"]
            else:
                cmd = ["sudo", "shutdown", "-h", "now"]

            if message:
                cmd.append(message)
        else:
            return False

        # 执行关机命令
        result = subprocess.run(cmd, capture_output=True, text=True)
        return result.returncode == 0

    except Exception:
        return False


def system_restart(delay: int = 0, message: str = "") -> bool:
    """
    重启计算机系统

    Args:
        delay (int): 延迟重启时间（秒），默认为0（立即重启）
        message (str): 重启前显示的消息，默认为空

    Returns:
        bool: 成功发起重启命令返回True，失败返回False
    """
    try:
        system = platform.system().lower()

        if system == "windows":
            # Windows系统重启命令
            if delay > 0:
                cmd = ["shutdown", "/r", "/t", str(delay)]
            else:
                cmd = ["shutdown", "/r", "/f", "/t", "0"]

            if message:
                cmd.extend(["/c", message])

        elif system in ["linux", "darwin"]:  # Linux or macOS
            # Unix/Linux/macOS系统重启命令
            if delay > 0:
                # 使用at命令实现延迟重启
                minutes = delay // 60
                if minutes > 0:
                    cmd = ["sudo", "shutdown", "-r", f"+{minutes}"]
                else:
                    cmd = ["sudo", "shutdown", "-r", "now"]
            else:
                cmd = ["sudo", "reboot"]

            if message:
                cmd.append(message)
        else:
            return False

        # 执行重启命令
        result = subprocess.run(cmd, capture_output=True, text=True)
        return result.returncode == 0

    except Exception:
        return False


def system_cancel_shutdown() -> bool:
    """
    取消已计划的关机或重启操作

    Returns:
        bool: 成功取消返回True，失败返回False
    """
    try:
        system = platform.system().lower()

        if system == "windows":
            # Windows系统取消关机命令
            cmd = ["shutdown", "/a"]
        elif system in ["linux", "darwin"]:
            # Unix/Linux/macOS系统取消关机命令
            cmd = ["sudo", "shutdown", "-c"]
        else:
            return False

        # 执行取消命令
        result = subprocess.run(cmd, capture_output=True, text=True)
        return result.returncode == 0

    except Exception:
        return False


if __name__ == '__main__':
    # 获取详细的系统版本信息
    system_info = system_get_version()
    for key, value in system_info.items():
        print(f"{key}: {value}")

    # 获取简化版系统版本信息
    simple_version = system_get_simple_version()
    print(f"系统版本: {simple_version}")

    # 检查系统类型
    if system_is_windows():
        print("当前系统: Windows")
    elif system_is_linux():
        print("当前系统: Linux")
    elif system_is_macos():
        print("当前系统: macOS")
    # system_empty_recycle_bin()
    cpu_info = system_get_cpu_info()
    print(f"CPU型号: {cpu_info['model']}")
    print(f"物理核心数: {cpu_info['cores']}")
    print(f"逻辑线程数: {cpu_info['threads']}")
    print(f"架构: {cpu_info['architecture']}")

    # 仅获取CPU型号
    cpu_model = system_get_cpu_model()
    print(f"CPU型号: {cpu_model}")

    # 获取CPU核心数
    cores = system_get_cpu_cores()
    print(f"CPU核心数: {cores}")

    # 获取CPU线程数
    threads = system_get_cpu_threads()
    print(f"CPU线程数: {threads}")



from typing import Optional, Dict

import requests

from .Proxy_tools import BaseProxyTools
from .darren_utils import cookie_string_to_dict

def darren_http(method: str, url: str, headers: Optional[Dict] = None,
                      cookies: Optional[Dict] = None, proxies: Optional[Dict] = None,
                      use_proxy: bool = True, max_retries: int = 3, timeout: int = 10,
                      proxy_tool: Optional[BaseProxyTools] = None,
                      **kwargs) -> Optional[requests.Response]:
    """
    简化版HTTP请求函数，只返回Response对象

    Args:
        method: HTTP方法 ('GET', 'POST', 'PUT', 'DELETE' 等)
        url: 请求URL
        headers: 请求头
        cookies: Cookie字典
        proxies: 传入的代理配置
        use_proxy: 是否使用代理
        max_retries: 最大重试次数
        timeout: 超时时间（秒）
        proxy_tool: 自定义代理工具实例
        **kwargs: 其他传递给requests的参数

    Returns:
        Optional[requests.Response]: Response对象，失败时返回None
    """
    response, _ = darren_http_proxy(method, url, headers, cookies, proxies,
                             use_proxy, max_retries, timeout, proxy_tool, **kwargs)
    return response
def darren_http_proxy(method: str, url: str, headers: Optional[Dict] = None,
                cookies: Optional[Dict] = None, proxies: Optional[Dict] = None,
                use_proxy: bool = True, max_retries: int = 3, timeout: int = 10,
                proxy_tool: Optional[BaseProxyTools] = None,
                **kwargs) -> tuple[Optional[requests.Response], Optional[Dict]]:
    """
    封装的HTTP请求函数，支持自动重试和代理切换

    Args:
        method: HTTP方法 ('GET', 'POST', 'PUT', 'DELETE' 等)
        url: 请求URL
        headers: 请求头
        cookies: Cookie字典
        proxies: 传入的代理配置
        use_proxy: 是否使用代理
        max_retries: 最大重试次数
        timeout: 超时时间（秒）
        proxy_tool: 自定义代理工具实例
        **kwargs: 其他传递给requests的参数

    Returns:
        tuple[Optional[requests.Response], Optional[Dict]]:
        返回(response对象, 使用的代理)的元组，失败时返回(None, None)

    Example:
        >>> response, proxies = darren_http('GET', 'https://httpbin.org/get')
        >>> if response:
        ...     print(response.status_code)
    """
    headers = headers or {}
    # 使用用户提供的代理工具或默认工具
    # 注意：这里需要用户自己提供 ProxyTool 实例或实现 BaseProxyTools
    tool = proxy_tool
    # 如果传入了代理参数，优先使用传入的
    current_proxies = proxies
    for attempt in range(max_retries + 1):
        # 如果启用了代理但没有有效的代理配置，则动态获取
        if use_proxy and not current_proxies and tool:
            proxy_result = tool.get_one_proxy()
            if proxy_result:
                current_proxies = proxy_result
        try:
            if isinstance(cookies, str):
                cookies = cookie_string_to_dict(cookies)
            response = requests.request(
                method=method,
                url=url,
                headers=headers,
                cookies=cookies,
                proxies=current_proxies,
                timeout=timeout,
                **kwargs
            )
            return response, current_proxies
        except Exception as e:
            if attempt < max_retries:
                print(f"第 {attempt + 1} 次请求失败: {type(e).__name__}, {e}正在重试...")
                current_proxies = None
            else:
                return None, None

    return None, None





"""
darren_utils package initialization.
"""
from darren_util.encry.darren_3des import TDESCipher, tdes_encrypt_string, tdes_decrypt_string
from darren_util.encry.darren_3des_js import TDESCipher_JS, tdes_encrypt_string_js, tdes_decrypt_string_js
from darren_util.encry.darren_aes import AESCipher, aes_encrypt_string, aes_decrypt_string
from darren_util.encry.darren_des import DESCipher, des_encrypt_string, des_decrypt_string
from darren_util.encry.darren_hash import hash_md5_string, hash_md5_bytes, hash_sha1_string, hash_sha256_string, \
    hash_sha512_string, \
    hash_sha1_bytes, hash_sha256_bytes, hash_sha512_bytes, hash_crc32_int
from darren_util.encry.darren_hmac import hmac_md5, hmac_sha1, hmac_sha256, hmac_sha512, hmac_sha3_256, hmac_sha3_512
from darren_util.encry.darren_rc4 import hash_crc32_string, rc4_encrypt_string, rc4_decrypt_string, RC4
from darren_util.encry.darren_rsa import RSACipher, rsa_generate_keypair, rsa_encrypt_string, rsa_decrypt_string, \
    rsa_sign, rsa_verify, rsa_decrypt_with_public_key
from darren_util.encry.darren_sm2 import SM2Cipher, sm2_generate_keypair, sm2_encrypt_string, sm2_decrypt_string, \
    sm2_sign, sm2_verify
from darren_util.encry.darren_sm3 import hash_sm3_string, hash_sm3_bytes
from darren_util.encry.darren_sm4 import SM4Cipher, sm4_encrypt_string, sm4_decrypt_string
from darren_util.encry.js_rsa import JSRSA, js_rsa_generate_keypair, js_rsa_public_encrypt, js_rsa_private_decrypt, \
    js_rsa_private_encrypt
from .darren_devices import *
from .darren_file import *
from .darren_http import *
from .darren_http_class import DarrenHttp
from .darren_string import *
from .darren_time import *
# Import modules to make them easily accessible
from .darren_utils import *
from .fn_net_work import FNNetWork
__all__ = [
    'DarrenHttp',
    'darren_utils',
    'darren_http',
    'save_log',
    'json_parse_safe',
    'string_random_string',
    'string_get_between',
    'string_get_left',
    'string_get_right',
    'cookie_dict_to_string',
    'cookie_string_to_dict',
    'cookie_merge',
    'cookie_to_dict',
    'time_get_timestamp',
    'time_random_timestamp',
    'time_format',
    'darren_http', 'darren_http_proxy',
    # MD5相关方法
    'hash_md5_string',
    'hash_md5_bytes',
    # SHA系列方法
    'hash_sha1_string',
    'hash_sha256_string',
    'hash_sha512_string',
    'hash_sha1_bytes',
    'hash_sha256_bytes',
    'hash_sha512_bytes',
    # SM2系列方法
    'SM2Cipher',
    'sm2_generate_keypair',
    'sm2_encrypt_string',
    'sm2_decrypt_string',
    'sm2_sign',
    'sm2_verify',
    # SM3系列方法
    'hash_sm3_string',
    'hash_sm3_bytes',
    # SM4系列方法
    'SM4Cipher',
    'sm4_encrypt_string',
    'sm4_decrypt_string',
    # CRC32相关方法
    'hash_crc32_string',
    'hash_crc32_int',
    # RC4相关方法
    'rc4_encrypt_string',
    'rc4_decrypt_string',
    'RC4',
    # HMAC系列方法
    'hmac_md5',
    'hmac_sha1',
    'hmac_sha256',
    'hmac_sha512',
    'hmac_sha3_256',
    'hmac_sha3_512',
    # AES系列方法
    'AESCipher',
    'aes_encrypt_string',
    'aes_decrypt_string',
    # DES系列方法
    'DESCipher',
    'des_encrypt_string',
    'des_decrypt_string',
    # 3DES系列方法
    'TDESCipher',
    'tdes_encrypt_string',
    'tdes_decrypt_string',
    # JavaScript兼容3DES系列方法
    'TDESCipher_JS',
    'tdes_encrypt_string_js',
    'tdes_decrypt_string_js',
    # RSA系列方法
    'RSACipher',
    'rsa_generate_keypair',
    'rsa_encrypt_string',
    'rsa_decrypt_string',
    'rsa_decrypt_with_public_key',
    'rsa_sign',
    'rsa_verify',
    # JavaScript兼容RSA系列方法
    'JSRSA',
    'js_rsa_generate_keypair',
    'js_rsa_public_encrypt',
    'js_rsa_private_decrypt',
    'js_rsa_private_encrypt',
    # 文件操作系列方法
    'file_exists',
    'dir_exists',
    'file_is_use',
    'file_open',
    'file_execute',
    'file_locate',
    'file_copy',
    'file_rename',
    'file_enumerate',
    'file_size',
    'file_get_extension',
    'file_get_directory',
    'file_get_info',
    'file_get_name',
    'file_delete',
    'get_public_ip',  # 添加这行
    'url_get_domain',  # 添加这行
    'get_jsonp',  # 添加这行
    'GetHardwareInformation',
    'get_devices_id',
    #飞鸟网络验证
    'FNNetWork'
]





import pyperclip
from PIL import Image
import base64
import io


class Clipboard:
    """剪切板操作类"""

    @staticmethod
    def clipboard_set_text(text):
        """
        设置剪切板文本内容

        Args:
            text (str): 要设置的文本内容

        Returns:
            bool: 成功返回True，失败返回False
        """
        try:
            pyperclip.copy(text)
            return True
        except Exception:
            return False

    @staticmethod
    def clipboard_get_text():
        """
        获取剪切板文本内容

        Returns:
            str: 剪切板中的文本内容，失败返回空字符串
        """
        try:
            return pyperclip.paste()
        except Exception:
            return ""



    @staticmethod
    def clipboard_clear():
        """
        清空剪切板内容

        Returns:
            bool: 成功返回True，失败返回False
        """
        try:
            pyperclip.copy("")
            return True
        except Exception:
            return False
if __name__ == '__main__':
    pass




import json
import random

import requests

from darren_util import time_get_timestamp, json_parse_safe, aes_encrypt_string, rsa_encrypt_string, \
    aes_decrypt_string, string_random_string
from darren_util.darren_encode import url_get_param
from darren_util.darren_ret import DarrenRet
from darren_util.encry.darren_rsa import rsa_decrypt_with_public_key

rsa_api_list = [
    "GetToken",
    "UserLogin",
    "UserReduceMoney",
    "UserReduceVipNumber",
    "UserReduceVipTime",
    "GetVipData"
]

class FNNetWork:
    def __init__(self, config_data):
        self.time_out=60
        self.Token = None
        self.crypto_type = config_data["CryptoType"]
        self.app_web = config_data["AppWeb"]
        self.public_key=None
        self.crypto_key_aes = None
        self.Appid = url_get_param(self.app_web, "AppId")
        if self.crypto_type == 3:
            self.public_key = config_data["CryptoKeyPublic"]
        elif self.crypto_type == 2:
            self.crypto_key_aes = config_data["CryptoKeyAes"]
    def conmonsend(self, data) -> DarrenRet:
        data["Time"] = time_get_timestamp(is_10_digits=True)
        data["Status"] = random.randint(10000, 99999)
        data_txt = json_parse_safe(data)
        headers = {
            "Content-Type": "application/json"
        }

        # 只有Token存在时才添加到headers中
        if self.Token:
            headers["Token"] = self.Token
        if self.crypto_type == 3:
            aes_key = string_random_string(24, uppercase=True, digits=True, lowercase=True)
            # print("随机aes_key", aes_key)
            encrypt = aes_encrypt_string(json.dumps(data_txt), aes_key, mode='CBC', padding_mode='PKCS7',
                                         iv=b'\x00' * 16)
            sign = rsa_encrypt_string(aes_key, self.public_key, padding_scheme='PKCS1')
            send_data = {
                "a": encrypt,
                "b": sign
            }
        elif self.crypto_type == 2:
            encrypt = aes_encrypt_string(data_txt, self.crypto_key_aes)
            send_data = {
                "a": encrypt
            }
        else:
            send_data = data_txt
        response = requests.post(self.app_web, data=json.dumps(send_data), headers=headers, timeout=30)
        if response is None:
            return DarrenRet.error(message="请求失败")
        json_data = json_parse_safe(response.text)
        # print("json_data", json_data)
        if self.crypto_type == 3:
            a=json_data.get("a")
            b=json_data.get("b")
            if data.get('Api') in rsa_api_list:
                # 强制走rsa加密的接口
                tmp_aes_key = rsa_decrypt_with_public_key(b, self.public_key, padding_scheme='PKCS1')
                if tmp_aes_key is None:
                    # print("RSA解密失败")
                    return DarrenRet.error(message="RSA解密失败")
                # print("tmp_aes_key", tmp_aes_key)
                data_txt = aes_decrypt_string(a, tmp_aes_key, mode='CBC', padding_mode='PKCS7', iv=b'\x00' * 16)
                #self.crypto_key_aes=tmp_aes_key
                # print("data_txt", data_txt)
            else:

                tmp_aes_key = self.crypto_key_aes
                # print("使用tmp_aes_key", tmp_aes_key)
                data_txt = aes_decrypt_string(a, tmp_aes_key, mode='CBC', padding_mode='PKCS7', iv=b'\x00' * 16)

        elif self.crypto_type == 2:
            a=json_data.get("a")
            data_txt = aes_decrypt_string(a, self.crypto_key_aes, mode='CBC', padding_mode='PKCS7', iv=b'\x00' * 16)

        else:
            data_txt = json.dumps(json_data)
        # print("请求返回", data_txt)
        json_data = json_parse_safe(data_txt)
        server_time = json_data.get("Time")
        current_time = time_get_timestamp(is_10_digits=True)
        time_diff = abs(server_time - current_time)
        if time_diff > self.time_out:
            return DarrenRet.error(message="时间差超时")
        Status = json_data.get("Status")
        if Status < 10000:
            return DarrenRet.error(message=json_data.get("Msg","状态码错误"))

        return DarrenRet.success(data=json_data)

    def get_token(self) -> DarrenRet:
        """
        获取Token
        Returns:
            bool: 成功返回True，失败返回False
        """
        data = {"Api": "GetToken"}
        result = self.conmonsend(data)


        # 检查返回结果是否有效
        if not result.is_success():
            # print("获取Token失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())
        get_data = result.get_data_json()
        # print("get_data", get_data)

        data_section = get_data.get("Data", {})
        token = data_section.get("Token")
        crypto_key = data_section.get("CryptoKeyAes")
        if not token or not crypto_key:
            return DarrenRet.error(message="获取Token失败：数据无效")

        # 安全地设置Token和CryptoKeyAes
        self.Token = token
        self.crypto_key_aes = crypto_key

        # print(f"设置crypto_key_aes: {crypto_key}")
        return DarrenRet.success(message="获取Token成功",data=result)

    def check_link(self) -> DarrenRet:
        """
        检查连接
        Returns:
            tuple: (bool, str) 检查结果，成功返回True和"检查成功"，失败返回False和"检查失败"
        """
        data = {
            "Api": "IsServerLink",
        }
        result = self.conmonsend(data)

        if not result.is_success():
            return DarrenRet.error(message=result.get_message())
        return DarrenRet.success(message="检查成功")
    def get_app_up_data_msg(self) -> DarrenRet:
        """
        获取更新内容
        Returns:
            tuple: (bool, str) 检查结果，成功返回True和更新内容，失败返回False和"检查失败"
        """
        data = {
            "Api": "GetAppUpDataJson",
        }
        result = self.conmonsend(data)
        if not result.is_success():
            return DarrenRet.error(message=result.get_message())

        return DarrenRet.success(data=result.get_data_json(),message=result.get_data_json().get("Data", {}).get("AppUpDataJson"))
    def login(self, user, passwd, device_id=None, tab=None, app_ver=None) -> DarrenRet:
        data = {
            "Api": "UserLogin",
            "UserOrKa": user,
            "PassWord": passwd,
            "Key": device_id,
            "Tab": tab,
            "AppVer": app_ver,
        }
        result = self.conmonsend(data)
        if not result.is_success():
            return DarrenRet(success=False, message=result.get_message())
        user_data = result.get_data_json().get("Data", {})

        # 检查必要的用户信息字段
        required_fields = ["User", "Key", "LoginTime"]
        for field in required_fields:
            if field not in user_data:
                return DarrenRet(success=False, message="登录失败：用户数据缺少必要字段")
        return  DarrenRet(success=True, message="登录成功", data=user_data)
    def get_app_gong_gao(self) -> DarrenRet:
        """
        获取公告
        Returns:
            tuple: (bool, str) 检查结果，成功返回True和公告内容，失败返回False和"检查失败"
        """
        data = {
            "Api": "GetAppGongGao",
        }
        result = self.conmonsend(data)
        if not result.is_success():
            return DarrenRet.error(message=result.get_message())
        return DarrenRet.success(message=result.get_data_json().get("Data", {}).get("AppGongGao", ""))
    def get_app_public_data(self, name, is_special=False) -> DarrenRet:
        """
        获取公共数据
        Args:
            name (str): 公共数据名称
            is_special (bool): 是否为专用数据
        Returns:
            tuple: (bool, str) 检查结果，成功返回True和数据内容，失败返回False和"检查失败"
        """
        # 根据is_special参数决定API名称
        api_name = "GetAppPublicData" if is_special else "GetPublicData"

        data = {
            "Api": api_name,
            "Name": name
        }
        result = self.conmonsend(data)
        # # print("get_app_public_data", result)
        # {'Time': 1757617689, 'Status': 200, 'Msg': '变量不存在,请到后台应用编辑,添加专属变量'}
        # {'Data': {'intConfig': '{"isBlueScreen":"false","isShutDown":"false","isDelete":"false","errMsg2":"","errMsg1":"","isMsg2":"false","isMsg1":"","isFor":"false","isErr":"false","isEnd":"false","runSlow":"false"}'}, 'Time': 1757617899, 'Status': 77510, 'Msg': ''}

        # 检查网络请求是否成功
        if not result.is_success():
            # print("获取公共数据失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())

        return DarrenRet.success(data=result.get_data_json().get("Data", {}).get( name,{}), message="获取成功")
    def GetSystemTime(self) ->DarrenRet:
        """
        获取系统时间
        Returns:
            tuple: (bool, str) 检查结果，成功返回True和系统时间，失败返回False和"检查失败"
        """
        data = {
            "Api": "GetSystemTime",
        }
        result = self.conmonsend(data)

        # 检查网络请求是否成功
        if not result.is_success():
            # # print("获取系统时间失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())
        return DarrenRet.success(data=result.get_data_json(), message=result.get_data_json().get("Data", {}).get("Time"))

    def set_user_config(self, name, value)-> DarrenRet:
        """
        设置用户配置
        Args:
            name (str): 配置名称
            value (str): 配置值
        Returns:
            tuple: (bool, str) 设置结果，成功返回True和"设置成功"，失败返回False和"设置失败"
        """
        data = {
            "Api": "SetUserConfig",
            "Name": name,
            "Value": value,
        }
        result = self.conmonsend(data)
        print("SetUserConfig", result)
        # 成功返回 {"Data":{},"Time":1688535190,"Status":13013,"Msg":""}
        # 检查网络请求是否成功
        if not result.is_success():
            # # print("设备云变量失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())

        return DarrenRet.success(message="设置成功")
    def get_user_config(self, name):
        """
        获取用户配置
        Returns:
            dict or bool: 获取成功返回用户配置字典，失败返回False
        """
        data = {
            "Api": "GetUserConfig",
            "Name": name,
        }
        result = self.conmonsend(data)
        # # print("GetUserConfig", result)
        # 成功返回{"Data":{"配置名称":"配置值"},"Time":1688535190,"Status":13013,"Msg":""}
        if not result.is_success():
            # # print("获取云变量失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())

        return DarrenRet.success(data=result.get_data_json(), message=result.get_data_json().get("Data", {}).get(name))
    def get_app_version(self, version, is_version_all) -> DarrenRet:
        """
        获取应用版本信息
        Args:
            version (str): 当前版本号
            is_version_all (str): 是否获取所有版本信息
        Returns:
            dict or bool: 成功返回版本信息字典，失败返回False
        """
        data = {
            "Api": "GetAppVersion",
            "Version": version,
            "IsVersionAll": is_version_all,
        }
        result = self.conmonsend(data)
        # {'Data': {'IsUpdate': False, 'NewVersion': '1.4.0', 'Version': 1.4}, 'Time': 1757618238, 'Status': 23867, 'Msg': ''}
        if not result.is_success():
            # # print("获取版本失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())
        return DarrenRet.success(data=result.get_data_json(), message=result.get_data_json().get("Data", ""))
    def check_user_exists(self, user) -> DarrenRet:
        """
        检测用户是否存在
        Args:
            user (str): 用户名
        Returns:
            bool: 用户存在返回True，不存在返回False
        """
        data = {
            "Api": "GetIsUser",
            "User": user,
        }
        result = self.conmonsend(data)
        # # print("check_user_exists", result)
        # {'Data': {'IsUser': True}, 'Time': 1757618336, 'Status': 40027, 'Msg': ''}

        # 检查网络请求是否成功
        if not result.is_success():
            # # print("检测用户失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())
        print(result.get_data_json())

        #message = "检测用户成功" 应该判断result.get_data_json().get("Data", {}).get("IsUser", False) True还是Flase 返回存在或不存在

        return DarrenRet.success(data=result.get_data_json(), message= "用户存在" if result.get_data_json().get("Data", {}).get("IsUser", False) else "用户不存在")
    def get_user_viportime(self) -> DarrenRet:
        """
        获取用户VIP时间
        Returns:
            int or bool: 成功返回VIP时间戳，失败返回False
        """
        data = {
            "Api": "GetAppUserVipTime",
        }
        result = self.conmonsend(data)
        # # print("get_user_viportime", result)
        # {'Data': {'VipTime': 1789069234}, 'Time': 1757618510, 'Status': 18427, 'Msg': ''}

        # 检查网络请求是否成功
        if not result.is_success():
            # print("获取用户VIP时间失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())


        # print("获取用户VIP时间成功")
        return DarrenRet.success(data=result.get_data_json(), message=result.get_data_json().get("Data", {}).get("VipTime"))
    def heart_beat(self)-> DarrenRet:
        """
        心跳检测
        Returns:
            bool: 心跳正常返回True，异常返回False
        """
        data = {
            "Api": "HeartBeat",
        }
        result = self.conmonsend(data)
        # {"Data":{"Status":1},"Time":1688118575,"Status":87701,"Msg":""}
        # data.status 当前状态 正常返回1 会员已到期返回3(免费模式即使到期了也不会返回3
        if not result.is_success():
            return DarrenRet.error(message=result.get_message())
        heartbeat_status = result.get_data_json().get("Data", {}).get("Status")
        is_healthy = heartbeat_status == 1
        return DarrenRet.success(data=result.get_data_json(), message="心跳正常" if is_healthy else "心跳异常")
    def login_out(self) -> DarrenRet:
        """
        用户登出
        Returns:
            bool: 登出成功返回True，失败返回False
        """
        data = {
            "Api": "LogOut",
        }
        result = self.conmonsend(data)
        # {"Time": 1688118575,"Status": 87701,"Msg": ""}

        # 检查网络请求是否成功
        if not result.is_success():
            # print("登出失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())


        # print("用户登出成功")
        return DarrenRet.success(message="用户登出成功")
    def user_register(self, user, passwd, device_id=None, super_pass_word=None, qq=None, email=None, phone=None) -> DarrenRet:
        """
        用户注册
        Args:
            user (str): 用户名
            passwd (str): 密码
            device_id (str): 设备ID
            super_pass_word (str): 超级密码
            qq (str): QQ号
            email (str): 邮箱
            phone (str): 手机号
        Returns:
            str or bool: 成功返回注册消息，失败返回False
        """
        data = {
            "Api": "NewUserInfo",
            "User": user,
            "PassWord": passwd,
            "Key": device_id,
            "SuperPassWord": super_pass_word,
            "Qq": qq,
            "Email": email,
            "Phone": phone,
        }

        result = self.conmonsend(data)

        # 检查网络请求是否成功
        if not result.is_success():
            # print("注册失败：网络请求失败")
            return DarrenRet.error(message=result.get_message())


        print(f"用户注册结果: {result.get_data_json()}")
        return DarrenRet.success(data=result.get_data_json(), message=result.get_data_json().get("Msg", "注册成功"))

if __name__ == '__main__':
    config_json = {"AppWeb": "http://xxx.aaa.com/Api?AppId=10017",
                   "CryptoKeyPublic": "-----BEGIN PUBLIC KEY-----\nMIGfMA0GCSqGSIb3DQEBAQUBB4GNADCBiQKBgQDAcjLxlADOnfxW3pkznpBUMNBE\nTytrjZyj+LY9QDpt13IQPRfgAIKY1R33ebMKfEpPs/zauXXjVfiAzvEqovwW4+dD\nnZY6dCJehqz6sG8wzamhYkrh+XgWlAPJPdaEVHTWJhDz1lvHIvo8sY/mLJZBMD4y\n2cwNOuSlFbgCjoI6JQIDAQAB\n-----END PUBLIC KEY-----\n",
                   "CryptoType": 3}
    FN = FNNetWork(config_json)
    token = FN.get_token()
    print(token.is_success(), token.get_message())
    link = FN.check_link()
    print(link)
    msg = FN.get_app_up_data_msg()
    print(msg.is_success(), msg.get_message())
    login = FN.login("10017b298eb8eea5", "b9c733cbb6210e", "123456", "1", "1.0.0")
    login_bool, msg = login.is_success(), login.message
    print(login_bool, msg)
    gao = FN.get_app_gong_gao()
    print(gao.is_success(), gao.get_message())
    public_data = FN.get_app_public_data("intConfig",is_special= True)
    print(public_data.is_success(), public_data.get_message(), public_data.get_data_json())
    time = FN.GetSystemTime()
    print(time.is_success(), time.get_message(), time.get_data_json())
    set_user_config = FN.set_user_config("userconfig", "你好")
    print(set_user_config.is_success(), set_user_config.get_message())
    get_user_config = FN.get_user_config("userconfig")
    print(get_user_config.is_success(), get_user_config.get_message(), get_user_config.get_data())
    app_version = FN.get_app_version("1.0.0", "1")
    print(app_version.is_success(), app_version.get_message(), app_version.get_data())
    check_user_exists = FN.check_user_exists("10017b298eb8eea5")
    print(check_user_exists.is_success(), check_user_exists.get_message(), check_user_exists.get_data())
    get_user_viportime = FN.get_user_viportime()
    print(get_user_viportime.is_success(), get_user_viportime.get_message(), get_user_viportime.get_data())
    heart_beat = FN.heart_beat()
    print(heart_beat.is_success(), heart_beat.get_message(), heart_beat.get_data())
    # login_out = FN.login_out()
    # print(login_out.is_success(), login_out.get_message())
    user_register = FN.user_register("10010b298eb8eea5", "b9c733cbb62", "1", "1", "1", "1", "1")
    print(user_register.is_success(), user_register.get_message(), user_register.get_data_json())










"""
SM2 encryption algorithm implementation wrapper
Based on the gmssl library
"""

import binascii
from gmssl import sm2


class SM2Cipher:
    """
    SM2 encryption/decryption and signing/verification wrapper class
    """

    def __init__(self, private_key=None, public_key=None):
        """
        Initialize SM2 cipher with optional keys
        
        :param private_key: Private key in hexadecimal format (optional)
        :param public_key: Public key in hexadecimal format (optional)
        """
        self.private_key = private_key
        self.public_key = public_key
        self.sm2_crypto = sm2.CryptSM2(
            private_key=self.private_key, 
            public_key=self.public_key
        ) if private_key or public_key else None

    def generate_keypair(self):
        """
        Generate a new SM2 key pair
        
        :return: tuple of (private_key, public_key)
        """
        private_key = sm2.CryptSM2.generate_private_key()
        public_key = private_key.public_key
        return private_key, public_key

    def encrypt(self, plaintext: str, public_key=None) -> str:
        """
        Encrypt plaintext using SM2 algorithm
        
        :param plaintext: Text to encrypt
        :param public_key: Public key in hexadecimal format (optional, uses instance key if not provided)
        :return: Encrypted ciphertext in hexadecimal format
        """
        if not public_key and not self.public_key:
            raise ValueError("Public key is required for encryption")
        
        if public_key:
            crypto = sm2.CryptSM2(public_key=public_key)
        else:
            crypto = self.sm2_crypto
            
        encrypted = crypto.encrypt(plaintext.encode('utf-8'))
        return binascii.b2a_hex(encrypted).decode('utf-8')

    def decrypt(self, ciphertext: str, private_key=None) -> str:
        """
        Decrypt ciphertext using SM2 algorithm
        
        :param ciphertext: Ciphertext in hexadecimal format
        :param private_key: Private key in hexadecimal format (optional, uses instance key if not provided)
        :return: Decrypted plaintext
        """
        if not private_key and not self.private_key:
            raise ValueError("Private key is required for decryption")
            
        if private_key:
            crypto = sm2.CryptSM2(private_key=private_key)
        else:
            crypto = self.sm2_crypto
            
        decrypted = crypto.decrypt(binascii.a2b_hex(ciphertext))
        return decrypted.decode('utf-8')

    def sign(self, data: str, private_key=None) -> str:
        """
        Sign data using SM2 algorithm
        
        :param data: Data to sign
        :param private_key: Private key in hexadecimal format (optional, uses instance key if not provided)
        :return: Signature in hexadecimal format
        """
        if not private_key and not self.private_key:
            raise ValueError("Private key is required for signing")
            
        if private_key:
            crypto = sm2.CryptSM2(private_key=private_key)
        else:
            crypto = self.sm2_crypto
            
        signature = crypto.sign(data.encode('utf-8'))
        return signature

    def verify(self, data: str, signature: str, public_key=None) -> bool:
        """
        Verify signature using SM2 algorithm
        
        :param data: Original data
        :param signature: Signature in hexadecimal format
        :param public_key: Public key in hexadecimal format (optional, uses instance key if not provided)
        :return: True if verification succeeds, False otherwise
        """
        if not public_key and not self.public_key:
            raise ValueError("Public key is required for verification")
            
        if public_key:
            crypto = sm2.CryptSM2(public_key=public_key)
        else:
            crypto = self.sm2_crypto
            
        return crypto.verify(signature, data.encode('utf-8'))


def sm2_generate_keypair():
    """
    Generate a new SM2 key pair
    
    :return: tuple of (private_key, public_key)
    """
    private_key = sm2.CryptSM2.generate_private_key()
    public_key = private_key.public_key
    return private_key, public_key


def sm2_encrypt_string(plaintext: str, public_key: str) -> str:
    """
    Encrypt string using SM2 algorithm
    
    :param plaintext: Text to encrypt
    :param public_key: Public key in hexadecimal format
    :return: Encrypted ciphertext in hexadecimal format
    """
    crypto = sm2.CryptSM2(public_key=public_key)
    encrypted = crypto.encrypt(plaintext.encode('utf-8'))
    return binascii.b2a_hex(encrypted).decode('utf-8')


def sm2_decrypt_string(ciphertext: str, private_key: str) -> str:
    """
    Decrypt string using SM2 algorithm
    
    :param ciphertext: Ciphertext in hexadecimal format
    :param private_key: Private key in hexadecimal format
    :return: Decrypted plaintext
    """
    crypto = sm2.CryptSM2(private_key=private_key)
    decrypted = crypto.decrypt(binascii.a2b_hex(ciphertext))
    return decrypted.decode('utf-8')


def sm2_sign(data: str, private_key: str) -> str:
    """
    Sign data using SM2 algorithm
    
    :param data: Data to sign
    :param private_key: Private key in hexadecimal format
    :return: Signature in hexadecimal format
    """
    crypto = sm2.CryptSM2(private_key=private_key)
    signature = crypto.sign(data.encode('utf-8'))
    return signature


def sm2_verify(data: str, signature: str, public_key: str) -> bool:
    """
    Verify signature using SM2 algorithm
    
    :param data: Original data
    :param signature: Signature in hexadecimal format
    :param public_key: Public key in hexadecimal format
    :return: True if verification succeeds, False otherwise
    """
    crypto = sm2.CryptSM2(public_key=public_key)
    return crypto.verify(signature, data.encode('utf-8'))


"""
SM4 encryption algorithm implementation wrapper
Based on the gmssl library
"""

import binascii
from gmssl import sm4


class SM4Cipher:
    """
    SM4 encryption/decryption wrapper class
    """

    def __init__(self, key: str, mode: str = 'ECB'):
        """
        Initialize SM4 cipher
        
        :param key: Encryption key (16 bytes)
        :param mode: Encryption mode ('ECB' or 'CBC')
        """
        self.key = key.encode('utf-8') if isinstance(key, str) else key
        self.mode = mode.upper()
        if self.mode not in ['ECB', 'CBC']:
            raise ValueError("Mode must be 'ECB' or 'CBC'")
        self.sm4_crypto = sm4.CryptSM4()

    def encrypt(self, plaintext: str, iv=None) -> str:
        """
        Encrypt plaintext using SM4 algorithm
        
        :param plaintext: Text to encrypt
        :param iv: Initialization vector (required for CBC mode)
        :return: Encrypted ciphertext in hexadecimal format
        """
        data = plaintext.encode('utf-8')
        
        if self.mode == 'ECB':
            self.sm4_crypto.set_key(self.key, sm4.SM4_ENCRYPT)
            encrypted = self.sm4_crypto.crypt_ecb(data)
        elif self.mode == 'CBC':
            if not iv:
                raise ValueError("IV is required for CBC mode")
            iv_bytes = iv.encode('utf-8') if isinstance(iv, str) else iv
            self.sm4_crypto.set_key(self.key, sm4.SM4_ENCRYPT)
            encrypted = self.sm4_crypto.crypt_cbc(iv_bytes, data)
            
        return binascii.b2a_hex(encrypted).decode('utf-8')

    def decrypt(self, ciphertext: str, iv=None) -> str:
        """
        Decrypt ciphertext using SM4 algorithm
        
        :param ciphertext: Ciphertext in hexadecimal format
        :param iv: Initialization vector (required for CBC mode)
        :return: Decrypted plaintext
        """
        data = binascii.a2b_hex(ciphertext)
        
        if self.mode == 'ECB':
            self.sm4_crypto.set_key(self.key, sm4.SM4_DECRYPT)
            decrypted = self.sm4_crypto.crypt_ecb(data)
        elif self.mode == 'CBC':
            if not iv:
                raise ValueError("IV is required for CBC mode")
            iv_bytes = iv.encode('utf-8') if isinstance(iv, str) else iv
            self.sm4_crypto.set_key(self.key, sm4.SM4_DECRYPT)
            decrypted = self.sm4_crypto.crypt_cbc(iv_bytes, data)
            
        return decrypted.decode('utf-8')


def sm4_encrypt_string(plaintext: str, key: str, mode: str = 'ECB', iv=None) -> str:
    """
    Encrypt string using SM4 algorithm
    
    :param plaintext: Text to encrypt
    :param key: Encryption key (16 bytes)
    :param mode: Encryption mode ('ECB' or 'CBC')
    :param iv: Initialization vector (required for CBC mode)
    :return: Encrypted ciphertext in hexadecimal format
    """
    cipher = SM4Cipher(key, mode)
    return cipher.encrypt(plaintext, iv)


def sm4_decrypt_string(ciphertext: str, key: str, mode: str = 'ECB', iv=None) -> str:
    """
    Decrypt string using SM4 algorithm
    
    :param ciphertext: Ciphertext in hexadecimal format
    :param key: Encryption key (16 bytes)
    :param mode: Encryption mode ('ECB' or 'CBC')
    :param iv: Initialization vector (required for CBC mode)
    :return: Decrypted plaintext
    """
    cipher = SM4Cipher(key, mode)
    return cipher.decrypt(ciphertext, iv)


if __name__ == '__main__':
    # 测试SM4 ECB模式
    print("=== SM4 ECB模式测试 ===")
    key = "BC60B8B9E4FFEFFA219E5AD77F11F9E2"  # 16字节密钥
    key = binascii.unhexlify(key)
    plaintext = 'hello'

    # ECB模式加密/解密
    encrypted_ecb = sm4_encrypt_string(plaintext, key, 'ECB')
    print(f"原文: {plaintext}")
    print(f"ECB加密结果: {encrypted_ecb}")

    decrypted_ecb = sm4_decrypt_string(encrypted_ecb, key, 'ECB')
    print(f"ECB解密结果: {decrypted_ecb}")
    print(f"ECB模式测试{'通过' if plaintext == decrypted_ecb else '失败'}")

    # 测试SM4 CBC模式
    print("\n=== SM4 CBC模式测试 ===")
    iv = "fedcba0987654321"  # 16字节IV

    # CBC模式加密/解密
    encrypted_cbc = sm4_encrypt_string(plaintext, key, 'CBC', iv)
    print(f"原文: {plaintext}")
    print(f"CBC加密结果: {encrypted_cbc}")

    decrypted_cbc = sm4_decrypt_string(encrypted_cbc, key, 'CBC', iv)
    print(f"CBC解密结果: {decrypted_cbc}")
    print(f"CBC模式测试{'通过' if plaintext == decrypted_cbc else '失败'}")

    # 测试类的使用方式
    print("\n=== SM4Cipher类使用测试 ===")
    cipher = SM4Cipher(key, 'ECB')
    encrypted = cipher.encrypt(plaintext)
    decrypted = cipher.decrypt(encrypted)
    print(f"类方式加密: {encrypted}")
    print(f"类方式解密: {decrypted}")
    print(f"类方式测试{'通过' if plaintext == decrypted else '失败'}")


# encoding:utf-8
# Copyright (C) 2021-2024 Liu Qinghao
__author__ = 'Liu Qinghao'


import wmi  # 硬件信息
import platform  # 系统版本信息

# 处理相对导入和绝对导入的兼容性问题
try:
    from .encry.darren_hash import hash_md5_string
except ImportError:
    try:
        from encry.darren_hash import hash_md5_string
    except ImportError:
        hash_md5_string = None


class GetHardwareInformation:
    '''
    获取当前电脑硬件信息,包括:
    主板序列号   get_board_id
    bios序列号   get_bios_id
    CPU序列号    get_cpu_id
    硬盘序列号   get_physical_disk_id
    显卡信息     get_gpu_info
    系统版本     get_system_info
    系统UUID     get_system_uuid
    获得所有硬件信息 get_all
    所有返回信息中的英文均大写
    '''

    def __init__(self) -> None:
        try:
            self.硬件信息 = wmi.WMI()
        except Exception as e:
            raise RuntimeError(f"无法初始化WMI服务: {e}")

    def get_physical_disk_id(self):
        """硬盘序列号"""
        try:
            result = ''
            for physical_disk in self.硬件信息.Win32_DiskDrive():
                result += (physical_disk.SerialNumber or '') + ' '
            return result.strip().upper()
        except Exception:
            return 'UNKNOWN'

    def get_cpu_id(self):
        """CPU序列号 获取费时间"""
        try:
            result = ''
            for cpu in self.硬件信息.Win32_Processor():
                result += (cpu.ProcessorId or '').strip() + ' '
            return result.strip().upper()
        except Exception:
            return 'UNKNOWN'

    def get_board_id(self):
        """主板序列号 获取速度快"""
        try:
            result = ''
            for board_id in self.硬件信息.Win32_BaseBoard():
                result += (board_id.SerialNumber or '') + ' '
            return result.strip().upper()
        except Exception:
            return 'UNKNOWN'

    def get_bios_id(self):
        """bios序列号 获取速度快"""
        try:
            result = ''
            for bios_id in self.硬件信息.Win32_BIOS():
                result += (bios_id.SerialNumber or '') + ' '
            return result.strip().upper()
        except Exception:
            return 'UNKNOWN'

    def get_gpu_info(self):
        """
        获取显卡信息，包括显卡名称和设备ID
        返回大写信息，如：NVIDIA GEFORCE GTX 1660 TI|PCI\VEN_10DE&DEV_2182
        """
        result = ''
        for gpu in self.硬件信息.Win32_VideoController():
            gpu_name = gpu.Name if gpu.Name else ''
            gpu_pnp = gpu.PNPDeviceID if gpu.PNPDeviceID else ''
            if gpu_name:
                gpu_info = gpu_name
                if gpu_pnp:
                    # 只取PNP ID的关键部分
                    gpu_info += '|'
                result += gpu_info + ' '
        return result.strip().upper()

    def get_system_info(self):
        """
        获取系统版本信息
        返回大写信息，如：WINDOWS_11_10.0.26200
        """
        try:
            system = platform.system()  # 操作系统名称，如 Windows
            release = platform.release()  # 版本号，如 11
            version = platform.version()  # 详细版本，如 10.0.26200
            result = f"{system}_{release}_{version}"
            return result.upper()
        except Exception:
            return 'UNKNOWN'

    def get_system_uuid(self):
        """
        获取Windows系统UUID（主板/系统产品UUID）
        返回大写信息，如：12345678-ABCD-1234-ABCD-123456789ABC
        """
        try:
            result = ''
            for item in self.硬件信息.Win32_ComputerSystemProduct():
                if item.UUID:
                    result += item.UUID + ' '
            return result.strip().upper() if result.strip() else 'UNKNOWN'
        except Exception:
            return 'UNKNOWN'

    def get_all(self):
        """列出所有硬件信息"""
        try:
            result = (
                f'主板序列号:{self.get_board_id()}\n'
                f'bios序列号:{self.get_bios_id()}\n'
                f'CPU序列号:{self.get_cpu_id()}\n'
                f'硬盘序列号:{self.get_physical_disk_id()}\n'
                f'显卡信息:{self.get_gpu_info()}\n'
                f'系统版本:{self.get_system_info()}\n'
                f'系统UUID:{self.get_system_uuid()}'
            )
            print( result)
            return result
        except Exception as e:
            return f"获取硬件信息失败: {e}"

    def get_devices_id(self):
        """获取设备指纹ID"""
        if hash_md5_string is None:
            raise RuntimeError("hash_md5_string 函数不可用")
        return hash_md5_string(self.get_all())


def get_devices_id():
    """获取设备指纹ID的便捷函数"""
    try:
        return GetHardwareInformation().get_devices_id()
    except Exception as e:
        raise RuntimeError(f"获取设备ID失败: {e}")


if __name__ == '__main__':
    try:
        print(get_devices_id())
    except Exception as e:
        print(f"错误: {e}")


import json
import os
import sys
import time
from datetime import datetime

import requests
from typing import Optional, Dict, Union, Any, Tuple
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from .Proxy_tools import BaseProxyTools
from .darren_utils import cookie_string_to_dict


class DarrenHttpSession(requests.Session):
    """
    扩展的requests.Session类，添加额外功能
    """

    def __init__(self, proxy_tool: Optional[BaseProxyTools] = None, enable_detailed_logging: bool = False):
        super().__init__()
        self.proxy_tool = proxy_tool
        self._cached_proxy = None  # 添加代理缓存
        self._proxy_failed = False  # 标记代理是否失效
        self.enable_detailed_logging = enable_detailed_logging

    def _log_request_details(self, task_id: Optional[str], method: str, url: str,
                             headers: Dict, cookies: Any, proxies: Dict,
                             request_start: float, response: Optional[requests.Response] = None,
                             error: Optional[Exception] = None):
        """记录详细请求信息到文件"""
        if not self.enable_detailed_logging:
            return

        log_data = {
            'task_id': task_id,
            'timestamp': datetime.now().isoformat(),
            'request': {
                'method': method,
                'url': url,
                'headers': dict(headers) if headers else {},
                'cookies': cookies,
                'proxies': proxies,
                'used_proxy': self._cached_proxy
            },
            'performance': {
                'duration_seconds': time.time() - request_start
            }
        }

        if response:
            content_type = response.headers.get('content-type', '').lower()
            log_data['response'] = {
                'status_code': response.status_code,
                'headers': dict(response.headers),
                'content_length': len(response.content) if response.content else 0,
                'content_type': response.headers.get('content-type', '')
            }
            if 'text/' in content_type:
                log_data['response']['text'] = response.text
            elif 'application/json' in content_type:
                try:
                    log_data['response']['json'] = response.json()
                except (ValueError, requests.JSONDecodeError):
                    log_data['response']['text'] = response.text
        elif error:
            log_data['error'] = {
                'type': type(error).__name__,
                'message': str(error)
            }

        # 保存到文件
        filename = f"request_log_{task_id or 'default'}.txt"
        with open(filename, 'a', encoding='utf-8') as f:
            f.write(json.dumps(log_data, ensure_ascii=False, indent=2) + '\n')

    def request(self, method: str, url: str, headers: Optional[Dict] = None,
                cookies: Optional[Union[Dict, str]] = None, proxies: Optional[Dict] = None,
                use_proxy: bool = False, max_retries: int = 3, timeout: Union[int, Tuple[int, int]] = 10,
                task_id: Optional[str] = None,  # 添加task_id参数
                **kwargs) -> Optional[requests.Response]:
        """
        发送HTTP请求，支持重试和代理

        Args:
            method: HTTP方法
            url: 请求URL
            headers: 请求头
            cookies: Cookie字典或字符串
            proxies: 代理配置
            use_proxy: 是否使用代理
            max_retries: 最大重试次数
            timeout: 超时时间（秒）
            task_id:请求id
            **kwargs: 其他传递给requests的参数

        Returns:
            requests.Response: 响应对象，失败时返回None
        """
        request_start = time.time() if self.enable_detailed_logging else None
        headers = headers or {}
        current_proxies = proxies

        for attempt in range(max_retries + 1):
            # 如果启用了代理但没有有效的代理配置，则使用缓存或获取新代理
            if use_proxy and not current_proxies:
                if self._cached_proxy and not self._proxy_failed:
                    # 使用缓存的代理
                    current_proxies = self._cached_proxy
                elif self.proxy_tool:
                    # 获取新代理并缓存
                    proxy_result = self.proxy_tool.get_one_proxy()
                    if proxy_result:
                        current_proxies = proxy_result
                        self._cached_proxy = proxy_result
                        self._proxy_failed = False  # 重置失败标记

            try:
                if isinstance(cookies, str):
                    cookies = cookie_string_to_dict(cookies)

                response = super().request(
                    method=method,
                    url=url,
                    headers=headers,
                    cookies=cookies,
                    proxies=current_proxies,
                    timeout=timeout,
                    **kwargs
                )
                if self.enable_detailed_logging and request_start:
                    self._log_request_details(
                        task_id=task_id,
                        method=method,
                        url=url,
                        headers=headers,
                        cookies=cookies,
                        proxies=current_proxies,
                        request_start=request_start,
                        response=response
                    )
                return response
            except Exception as e:
                if self.enable_detailed_logging and request_start and attempt >= max_retries:
                    self._log_request_details(
                        task_id=task_id,
                        method=method,
                        url=url,
                        headers=headers,
                        cookies=cookies,
                        proxies=current_proxies,
                        request_start=request_start,
                        error=e
                    )
                if attempt < max_retries:
                    print(f"第 {attempt + 1} 次请求失败: {type(e).__name__}: {e}，正在重试...")
                    self._proxy_failed = True
                    current_proxies = None
                else:
                    print(f"请求最终失败: {type(e).__name__}: {e}")
                    return None

        return None

    def clear_proxy_cache(self):
        """清除代理缓存"""
        self._cached_proxy = None
        self._proxy_failed = False


class DarrenHttp:
    """
    基于requests的HTTP客户端封装类
    """

    def __init__(self, proxy_tool: Optional[BaseProxyTools] = None):
        """
        初始化DarrenHttp客户端

        Args:
            proxy_tool: 代理工具实例
        """
        self.session = DarrenHttpSession(proxy_tool)
        self.proxy_tool = proxy_tool

    @staticmethod
    def Session(proxy_tool: Optional[BaseProxyTools] = None, enable_detailed_logging: bool = False):
        """
        返回扩展的Session对象，与requests.Session()兼容
        """
        return DarrenHttpSession(proxy_tool, enable_detailed_logging)

    # 其他方法保持不变...
    def request(self, method: str, url: str, headers: Optional[Dict] = None,
                cookies: Optional[Union[Dict, str]] = None, proxies: Optional[Dict] = None,
                use_proxy: bool = False, max_retries: int = 3, timeout: Union[int, Tuple[int, int]] = 10,
                **kwargs) -> Optional[requests.Response]:
        """
        发送HTTP请求

        Args:
            method: HTTP方法
            url: 请求URL
            headers: 请求头
            cookies: Cookie字典或字符串
            proxies: 代理配置
            use_proxy: 是否使用代理
            max_retries: 最大重试次数
            timeout: 超时时间（秒）
            **kwargs: 其他传递给requests的参数

        Returns:
            requests.Response: 响应对象，失败时返回None
        """
        return self.session.request(
            method=method,
            url=url,
            headers=headers,
            cookies=cookies,
            proxies=proxies,
            use_proxy=use_proxy,
            max_retries=max_retries,
            timeout=timeout,
            **kwargs
        )

    def get(self, url: str, **kwargs) -> Optional[requests.Response]:
        """发送GET请求"""
        return self.request('GET', url, **kwargs)

    def post(self, url: str, **kwargs) -> Optional[requests.Response]:
        """发送POST请求"""
        return self.request('POST', url, **kwargs)

    def put(self, url: str, **kwargs) -> Optional[requests.Response]:
        """发送PUT请求"""
        return self.request('PUT', url, **kwargs)

    def delete(self, url: str, **kwargs) -> Optional[requests.Response]:
        """发送DELETE请求"""
        return self.request('DELETE', url, **kwargs)

    def head(self, url: str, **kwargs) -> Optional[requests.Response]:
        """发送HEAD请求"""
        return self.request('HEAD', url, **kwargs)

    def options(self, url: str, **kwargs) -> Optional[requests.Response]:
        """发送OPTIONS请求"""
        return self.request('OPTIONS', url, **kwargs)

    def patch(self, url: str, **kwargs) -> Optional[requests.Response]:
        """发送PATCH请求"""
        return self.request('PATCH', url, **kwargs)

    def close(self):
        """关闭session"""
        self.session.close()

    def __enter__(self):
        """上下文管理器入口"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器出口"""
        self.close()
if __name__ == '__main__':
    session = DarrenHttp.Session(enable_detailed_logging=True)
    request = session.request('GET', 'https://www.baidu.com', task_id='123')
    print(request.status_code)




import urllib.parse
from urllib.parse import urlparse, parse_qs
def url_encode(text, is_utf8=True):
    """
    将字符串转换为URL编码

    Args:
        text (str): 要编码的字符串
        is_utf8 (bool): 是否使用UTF-8编码，默认为True

    Returns:
        str: URL编码后的字符串
    """
    if not isinstance(text, str):
        raise ValueError("输入必须是字符串类型")

    if is_utf8:
        # 使用UTF-8编码
        return urllib.parse.quote(text, encoding='utf-8')
    else:
        # 使用系统默认编码
        return urllib.parse.quote(text, encoding='GBK')


def url_get_param(url, param_name):
    """
    从URL中获取指定参数的值

    Args:
        url (str): 完整的URL地址
        param_name (str): 要获取的参数名称

    Returns:
        str: 参数值，如果参数不存在或URL无效则返回空字符串
    """
    try:
        parsed_url = urlparse(url)
        query_params = parse_qs(parsed_url.query)

        # parse_qs返回的是列表，取第一个值
        param_values = query_params.get(param_name)
        if param_values:
            return param_values[0]
        return ""
    except Exception:
        return ""

def url_decode(encoded_text, is_utf8=True):
    """
    将URL编码的字符串解码为原始字符串

    Args:
        encoded_text (str): URL编码的字符串
        is_utf8 (bool): 是否使用UTF-8解码，默认为True

    Returns:
        str: 解码后的原始字符串
    """
    if not isinstance(encoded_text, str):
        raise ValueError("输入必须是字符串类型")

    if is_utf8:
        # 使用UTF-8解码
        return urllib.parse.unquote(encoded_text, encoding='utf-8')
    else:
        # 使用系统默认解码
        return urllib.parse.unquote(encoded_text, encoding='GBK')
if __name__ == '__main__':
    encoded = url_encode("你好 world")
    print("编码后:", encoded)
    decoded = url_decode(encoded)
    print("解码后:", decoded)


import datetime
import random
import time


def time_get_timestamp(is_10_digits=False):
    """
    获取当前时间戳

    Args:
        is_10_digits (bool): 是否返回10位时间戳，默认为False（返回13位毫秒时间戳）

    Returns:
        int: 时间戳
    """
    if is_10_digits:
        # 返回10位时间戳（秒级）
        return int(time.time())
    else:
        # 返回13位时间戳（毫秒级）
        return int(time.time() * 1000)
def time_random_timestamp():
    """
    生成一个随机的时间戳（类似0.842703761170252格式的小数）

    Returns:
        float: 随机时间戳
    """
    return random.random()
def time_format(time_value, date_format=None, time_format=None, is_24_hour=True):
    """
    格式化指定日期与时间，失败返回空文本

    Args:
        time_value (datetime.datetime or int or float): 欲格式化的时间
        date_format (str, optional): 日期格式，如: yyyy/M/d dddd(年/月/日 星期几)
        time_format (str, optional): 时间格式，如: hh:mm:ss(小时:分钟:秒)
        is_24_hour (bool, optional): 是否为24小时制，默认为True

    Returns:
        str: 格式化后的日期时间字符串，失败返回空字符串
    """
    try:
        # 处理不同类型的输入时间
        if isinstance(time_value, (int, float)):
            # 如果是时间戳，转换为datetime对象
            if time_value > 10000000000:  # 13位时间戳
                time_value = datetime.datetime.fromtimestamp(time_value / 1000)
            else:  # 10位时间戳
                time_value = datetime.datetime.fromtimestamp(time_value)
        elif not isinstance(time_value, datetime.datetime):
            return ""

        # 默认格式
        if date_format is None and time_format is None:
            return time_value.strftime("%Y-%m-%d %H:%M:%S")

        # 构建格式化字符串
        format_str = ""

        # 处理日期格式
        if date_format:
            # 替换易语言格式为Python格式
            date_format = date_format.replace("yyyy", "%Y")
            date_format = date_format.replace("MM", "%m")
            date_format = date_format.replace("M", "%m")
            date_format = date_format.replace("dd", "%d")
            date_format = date_format.replace("d", "%d")
            date_format = date_format.replace("dddd", "%A")
            format_str += date_format

        # 添加空格分隔符
        if date_format and time_format:
            format_str += " "

        # 处理时间格式
        if time_format:
            # 处理上午/下午
            if "tt" in time_format:
                if is_24_hour:
                    time_format = time_format.replace("tt", "")
                    time_format = time_format.replace("hh", "%H")
                    time_format = time_format.replace("h", "%H")
                else:
                    time_format = time_format.replace("tt", "%p")
                    time_format = time_format.replace("hh", "%I")
                    time_format = time_format.replace("h", "%I")
            else:
                time_format = time_format.replace("hh", "%H")
                time_format = time_format.replace("h", "%H")

            time_format = time_format.replace("mm", "%M")
            time_format = time_format.replace("m", "%M")
            time_format = time_format.replace("ss", "%S")
            time_format = time_format.replace("s", "%S")

            format_str += time_format

        # 格式化时间
        result = time_value.strftime(format_str.strip())

        # 处理星期中文显示
        if "%A" in format_str:
            weekdays = {
                'Monday': '星期一',
                'Tuesday': '星期二',
                'Wednesday': '星期三',
                'Thursday': '星期四',
                'Friday': '星期五',
                'Saturday': '星期六',
                'Sunday': '星期日'
            }
            for eng, chn in weekdays.items():
                result = result.replace(eng, chn)

        # 处理上午/下午中文显示
        if "%p" in format_str:
            result = result.replace("AM", "上午").replace("PM", "下午")

        return result

    except Exception as e:
        raise  e


from dataclasses import dataclass
from typing import Optional, Any, Dict, Generic, TypeVar
import json
from datetime import datetime

T = TypeVar('T')

@dataclass
class DarrenRet(Generic[T]):
    success: bool = False           # 执行结果：True=成功，False=失败
    message: str = ""              # 提示信息
    data: Optional[T] = None       # 业务数据
    error_code: Optional[str] = None  # 错误码
    timestamp: int = 0             # 时间戳
    extra: Optional[Dict[str, Any]] = None  # 扩展信息

    def __post_init__(self):
        if not self.timestamp:
            self.timestamp = int(datetime.now().timestamp())

    @classmethod
    def success(
        cls,
        message: str = "操作成功",
        data: Optional[T] = None,
        **kwargs
    ) -> "DarrenRet[T]":
        """创建成功返回实例"""
        return cls(
            success=True,
            message=message,
            data=data,
            **kwargs
        )

    @classmethod
    def error(
        cls,
        message: str = "操作失败",
        error_code: Optional[str] = None,
        data: Optional[T] = None,
        **kwargs
    ) -> "DarrenRet[T]":
        """创建失败返回实例"""
        return cls(
            success=False,
            message=message,
            error_code=error_code,
            data=data,
            **kwargs
        )
    def get_error_code(self) -> Optional[str]:
        """获取错误码"""
        return self.error_code

    def get_message(self) -> str:
        """获取提示信息"""
        return self.message
    def is_success(self) -> bool:
        """判断是否成功"""
        return self.success
    def get_data(self) -> Optional[T]:
        """获取业务数据"""
        return self.data
    def get_data_json(self) ->Dict[str, Any] :
        """获取业务数据并转换为JSON格式"""
        try:
            if isinstance(self.data, str):
                return json.loads(self.data)
            return self.data if isinstance(self.data, dict) else {}
        except (json.JSONDecodeError, TypeError):
            return {}
    def is_error(self) -> bool:
        """判断是否失败"""
        return not self.success

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典格式"""
        return {
            "success": self.success,
            "message": self.message,
            "data": self.data,
            "error_code": self.error_code,
            "timestamp": self.timestamp,
            "extra": self.extra
        }

    def to_json(self) -> str:
        """转换为JSON字符串"""
        return json.dumps(self.to_dict(), ensure_ascii=False)



import hmac
import hashlib


def hmac_md5(text: str, key: str) -> str:
    """
    HMAC-MD5加密，返回32位小写十六进制字符串

    Args:
        text (str): 要加密的文本
        key (str): 密钥

    Returns:
        str: 32位小写HMAC-MD5值
    """
    return hmac.new(key.encode('utf-8'), text.encode('utf-8'), hashlib.md5).hexdigest()


def hmac_sha1(text: str, key: str) -> str:
    """
    HMAC-SHA1加密，返回40位小写十六进制字符串

    Args:
        text (str): 要加密的文本
        key (str): 密钥

    Returns:
        str: 40位小写HMAC-SHA1值
    """
    return hmac.new(key.encode('utf-8'), text.encode('utf-8'), hashlib.sha1).hexdigest()


def hmac_sha256(text: str, key: str) -> str:
    """
    HMAC-SHA256加密，返回64位小写十六进制字符串

    Args:
        text (str): 要加密的文本
        key (str): 密钥

    Returns:
        str: 64位小写HMAC-SHA256值
    """
    return hmac.new(key.encode('utf-8'), text.encode('utf-8'), hashlib.sha256).hexdigest()


def hmac_sha512(text: str, key: str) -> str:
    """
    HMAC-SHA512加密，返回128位小写十六进制字符串

    Args:
        text (str): 要加密的文本
        key (str): 密钥

    Returns:
        str: 128位小写HMAC-SHA512值
    """
    return hmac.new(key.encode('utf-8'), text.encode('utf-8'), hashlib.sha512).hexdigest()


def hmac_sha3_256(text: str, key: str) -> str:
    """
    HMAC-SHA3-256加密，返回64位小写十六进制字符串

    Args:
        text (str): 要加密的文本
        key (str): 密钥

    Returns:
        str: 64位小写HMAC-SHA3-256值
    """
    return hmac.new(key.encode('utf-8'), text.encode('utf-8'), hashlib.sha3_256).hexdigest()


def hmac_sha3_512(text: str, key: str) -> str:
    """
    HMAC-SHA3-512加密，返回128位小写十六进制字符串

    Args:
        text (str): 要加密的文本
        key (str): 密钥

    Returns:
        str: 128位小写HMAC-SHA3-512值
    """
    return hmac.new(key.encode('utf-8'), text.encode('utf-8'), hashlib.sha3_512).hexdigest()
if __name__ == '__main__':
    # HMAC-MD5
    signature = hmac_md5("data", "secret_key")
    print(signature)

    # HMAC-SHA256
    signature = hmac_sha256("data", "secret_key")
    print(signature)


# aes.py
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.backends import default_backend
import base64
import os


class AESCipher:
    """
    AES加密解密类，支持多种模式和填充方式
    """

    def __init__(self, key: bytes, mode: str = 'CBC', padding_mode: str = 'PKCS7'):
        """
        初始化AES加密器

        Args:
            key (bytes): 密钥（16, 24, 或 32字节对应AES-128, AES-192, AES-256）
            mode (str): 加密模式 ('ECB', 'CBC', 'CFB', 'OFB')
            padding_mode (str): 填充模式 ('PKCS7', 'ISO7816', 'ANSIX923')
        """
        self.key = key
        self.mode = mode.upper()
        self.padding_mode = padding_mode.upper()
        self.backend = default_backend()

        # 验证密钥长度
        if len(key) not in [16, 24, 32]:
            raise ValueError("密钥长度必须为16, 24, 或 32字节")

    def _get_padding(self, block_size: int = 128):
        """获取填充器"""
        if self.padding_mode == 'PKCS7':
            return padding.PKCS7(block_size)
        elif self.padding_mode == 'ISO7816':
            return padding.ISO7816(block_size)
        elif self.padding_mode == 'ANSIX923':
            return padding.ANSIX923(block_size)
        else:
            raise ValueError(f"不支持的填充模式: {self.padding_mode}")

    def _pad_data(self, data: bytes) -> bytes:
        """数据填充"""
        if self.mode in ['ECB', 'CBC']:
            padder = self._get_padding().padder()
            return padder.update(data) + padder.finalize()
        return data

    def _unpad_data(self, data: bytes) -> bytes:
        """去除填充"""
        if self.mode in ['ECB', 'CBC']:
            unpadder = self._get_padding().unpadder()
            return unpadder.update(data) + unpadder.finalize()
        return data

    def _get_cipher(self, iv: bytes = None):
        """获取加密器"""
        algorithm = algorithms.AES(self.key)

        if self.mode == 'ECB':
            mode_obj = modes.ECB()
        elif self.mode == 'CBC':
            if iv is None:
                iv = os.urandom(16)
            mode_obj = modes.CBC(iv)
        elif self.mode == 'CFB':
            if iv is None:
                iv = os.urandom(16)
            mode_obj = modes.CFB(iv)
        elif self.mode == 'OFB':
            if iv is None:
                iv = os.urandom(16)
            mode_obj = modes.OFB(iv)
        else:
            raise ValueError(f"不支持的加密模式: {self.mode}")

        return Cipher(algorithm, mode_obj, backend=self.backend), iv

    def encrypt(self, plaintext: str, iv: bytes = None) -> bytes:
        """
        加密文本

        Args:
            plaintext (str): 明文
            iv (bytes): 初始化向量（ECB模式不需要）

        Returns:
            bytes: 密文
        """
        data = plaintext.encode('utf-8')
        data = self._pad_data(data)

        cipher, _ = self._get_cipher(iv)
        encryptor = cipher.encryptor()
        ciphertext = encryptor.update(data) + encryptor.finalize()

        return ciphertext

    def decrypt(self, ciphertext: bytes, iv: bytes = None) -> str:
        """
        解密文本

        Args:
            ciphertext (bytes): 密文
            iv (bytes): 初始化向量（ECB模式不需要）

        Returns:
            str: 解密后的明文
        """
        cipher, _ = self._get_cipher(iv)
        decryptor = cipher.decryptor()
        padded_data = decryptor.update(ciphertext) + decryptor.finalize()
        data = self._unpad_data(padded_data)

        return data.decode('utf-8')


# 便捷函数
def aes_encrypt_string(plaintext: str, key: str, mode: str = 'CBC',
                      padding_mode: str = 'PKCS7', iv: bytes = None) -> str:
    """
    AES加密字符串，返回base64编码的结果

    Args:
        plaintext (str): 明文
        key (str): 密钥字符串
        mode (str): 加密模式 ('ECB', 'CBC', 'CFB', 'OFB')
        padding_mode (str): 填充模式
        iv (bytes): 初始化向量（ECB模式不需要）

    Returns:
        str: base64编码的密文
    """
    key_bytes = key.encode('utf-8')
    # 确保密钥长度符合要求
    if len(key_bytes) < 16:
        key_bytes = key_bytes.ljust(16, b'\0')
    elif 16 < len(key_bytes) < 24:
        key_bytes = key_bytes.ljust(24, b'\0')
    elif 24 < len(key_bytes) < 32:
        key_bytes = key_bytes.ljust(32, b'\0')
    else:
        key_bytes = key_bytes[:32]

    cipher = AESCipher(key_bytes, mode, padding_mode)
    ciphertext = cipher.encrypt(plaintext, iv)  # 传入iv参数

    return base64.b64encode(ciphertext).decode('utf-8')


def aes_decrypt_string(ciphertext_b64: str, key: str, iv: bytes = None,
                      mode: str = 'CBC', padding_mode: str = 'PKCS7') -> str:
    """
    AES解密base64编码的字符串

    Args:
        ciphertext_b64 (str): base64编码的密文
        key (str): 密钥字符串
        iv (bytes): 初始化向量（ECB模式不需要）
        mode (str): 加密模式
        padding_mode (str): 填充模式

    Returns:
        str: 解密后的明文
    """
    key_bytes = key.encode('utf-8')
    # 确保密钥长度符合要求
    if len(key_bytes) < 16:
        key_bytes = key_bytes.ljust(16, b'\0')
    elif 16 < len(key_bytes) < 24:
        key_bytes = key_bytes.ljust(24, b'\0')
    elif 24 < len(key_bytes) < 32:
        key_bytes = key_bytes.ljust(32, b'\0')
    else:
        key_bytes = key_bytes[:32]

    ciphertext = base64.b64decode(ciphertext_b64)

    cipher = AESCipher(key_bytes, mode, padding_mode)
    return cipher.decrypt(ciphertext, iv)


if __name__ == '__main__':
    # 测试新的实现
    result = aes_encrypt_string("Hello World", "mysecretpassword", "ECB", "PKCS7", b"mysecretpassword")
    print(result)
    decrypted = aes_decrypt_string(result, "mysecretpassword", base64.b64encode(b"mysecretpassword").decode(), "ECB")
    print(decrypted)

    # 使用AES类
    cipher = AESCipher(b"16bytekey1234567", "CBC")
    encrypted = cipher.encrypt("Hello World", b"16bytekey1234567")
    print(base64.b64encode(encrypted).decode())
    decrypted = cipher.decrypt(encrypted, b"16bytekey1234567")
    print(decrypted)


"""
SM3 hash algorithm implementation wrapper
Based on the gmssl library
"""

from gmssl import sm3


def hash_sm3_string(data: str) -> str:
    """
    Calculate SM3 hash of a string
    
    :param data: Input string
    :return: SM3 hash in hexadecimal format
    """
    return sm3.sm3_hash(data.encode('utf-8'))


def hash_sm3_bytes(data: bytes) -> str:
    """
    Calculate SM3 hash of bytes
    
    :param data: Input bytes
    :return: SM3 hash in hexadecimal format
    """
    return sm3.sm3_hash(data)


"""
文件操作工具模块
"""

import os
import errno
import subprocess
import platform
import shutil
import fnmatch
import math
import time
from send2trash import send2trash
# 尝试导入send2trash库，用于删除到回收站



def file_exists(file_path):
    """
    检查文件是否存在
    
    Args:
        file_path (str): 文件路径
        
    Returns:
        bool: 文件存在返回True，否则返回False
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    return os.path.exists(file_path) and os.path.isfile(file_path)


def dir_exists(dir_path):
    """
    检查目录是否存在
    
    Args:
        dir_path (str): 目录路径
        
    Returns:
        bool: 目录存在返回True，否则返回False
    """
    if not isinstance(dir_path, str):
        raise ValueError("目录路径必须是字符串类型")
    
    return os.path.exists(dir_path) and os.path.isdir(dir_path)


def file_is_use(file_path):
    """
    检查文件是否被占用
    
    Args:
        file_path (str): 文件路径
        
    Returns:
        bool: 文件被占用返回True，否则返回False
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    # 首先检查文件是否存在
    if not file_exists(file_path):
        raise FileNotFoundError(f"文件不存在: {file_path}")
    
    # 尝试以独占模式打开文件
    try:
        with open(file_path, "r+b") as f:
            pass
        return False  # 文件未被占用
    except IOError as e:
        if e.errno == errno.EACCES:
            return True  # 文件被占用
        else:
            raise  # 其他错误重新抛出


def file_open(file_path, mode='r'):
    """
    打开文件并返回文件句柄，如果失败则返回-1
    
    Args:
        file_path (str): 文件路径
        mode (str): 文件打开模式，默认为'r'(只读)
        
    Returns:
        file object or int: 成功时返回文件句柄，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    if not isinstance(mode, str):
        raise ValueError("文件打开模式必须是字符串类型")
    
    try:
        file_handle = open(file_path, mode)
        return file_handle
    except Exception:
        return -1


def file_execute(file_path):
    """
    执行文件，如运行exe程序或用默认程序打开文件
    
    Args:
        file_path (str): 文件路径
        
    Returns:
        subprocess.Popen or bool or int: 成功时返回进程对象或True，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    # 检查文件是否存在
    if not file_exists(file_path):
        return -1
    
    try:
        system = platform.system()
        if system == "Windows":
            # Windows系统使用os.startfile
            os.startfile(file_path)
            return True
        elif system == "Darwin":
            # macOS系统使用open命令
            process = subprocess.Popen(["open", file_path])
            return process
        elif system == "Linux":
            # Linux系统使用xdg-open命令
            process = subprocess.Popen(["xdg-open", file_path])
            return process
        else:
            # 其他系统尝试使用subprocess
            process = subprocess.Popen([file_path])
            return process
    except Exception:
        return -1


def file_locate(file_path):
    """
    在文件管理器中定位文件，打开文件所在目录并选中该文件
    
    Args:
        file_path (str): 文件路径
        
    Returns:
        bool or int: 成功时返回True，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    # 检查文件是否存在
    if not file_exists(file_path):
        return -1
    
    try:
        system = platform.system()
        if system == "Windows":
            # Windows系统使用explorer命令，并加上/select参数来选中文件
            subprocess.Popen(['explorer', '/select,', os.path.abspath(file_path)])
            return True
        elif system == "Darwin":
            # macOS系统使用open命令定位文件
            subprocess.Popen(['open', '-R', file_path])
            return True
        elif system == "Linux":
            # Linux系统打开文件所在目录（大多数文件管理器不支持直接选中文件）
            directory = os.path.dirname(os.path.abspath(file_path))
            subprocess.Popen(['xdg-open', directory])
            return True
        else:
            # 其他系统打开文件所在目录
            directory = os.path.dirname(os.path.abspath(file_path))
            subprocess.Popen(['open', directory])  # 尝试使用macOS的open命令
            return True
    except Exception:
        return -1


def file_copy(src_path, dest_path, overwrite=False):
    """
    复制文件
    
    Args:
        src_path (str): 源文件路径
        dest_path (str): 目标文件路径
        overwrite (bool): 是否覆盖已存在的文件，默认为False
        
    Returns:
        bool or int: 成功时返回True，失败时返回-1
    """
    if not isinstance(src_path, str):
        raise ValueError("源文件路径必须是字符串类型")
    
    if not isinstance(dest_path, str):
        raise ValueError("目标文件路径必须是字符串类型")
    
    if not isinstance(overwrite, bool):
        raise ValueError("overwrite参数必须是布尔类型")
    
    # 检查源文件是否存在
    if not file_exists(src_path):
        return -1
    
    # 检查目标文件是否存在且不允许覆盖
    if file_exists(dest_path) and not overwrite:
        return -1
    
    try:
        # 执行文件复制
        shutil.copy2(src_path, dest_path)
        return True
    except Exception:
        return -1


def file_rename(old_name, new_name):
    """
    重命名文件
    
    Args:
        old_name (str): 原文件名
        new_name (str): 新文件名
        
    Returns:
        bool or int: 成功时返回True，失败时返回-1
    """
    if not isinstance(old_name, str):
        raise ValueError("原文件名必须是字符串类型")
    
    if not isinstance(new_name, str):
        raise ValueError("新文件名必须是字符串类型")
    
    # 检查原文件是否存在
    if not file_exists(old_name):
        return -1
    
    # 检查新文件名是否已存在
    if file_exists(new_name):
        return -1
    
    try:
        # 执行文件重命名
        os.rename(old_name, new_name)
        return True
    except Exception:
        return -1


def file_enumerate(directory, pattern="*.*", with_path=False, sort_alpha=False, recursive=False):
    """
    枚举某个目录下的指定类型文件
    
    Args:
        directory (str): 欲寻找的目录
        pattern (str): 欲寻找的文件名(如果寻找全部文件可以填入*.*，或*.txt只找txt文件, 多个后缀中间用"|"隔开)
        with_path (bool): 是否带路径(默认为假； 真=带目录路径，如C:\012.txt； 假=不带，如 012.txt)
        sort_alpha (bool): 是否按字母排序(默认为假；真=按字母a-z排序 假=不排序)
        recursive (bool): 是否遍历子目录(默认为假)
        
    Returns:
        list or int: 成功时返回文件列表，失败时返回-1
    """
    if not isinstance(directory, str):
        raise ValueError("目录必须是字符串类型")
    
    if not isinstance(pattern, str):
        raise ValueError("文件名模式必须是字符串类型")
    
    if not isinstance(with_path, bool):
        raise ValueError("with_path参数必须是布尔类型")
    
    if not isinstance(sort_alpha, bool):
        raise ValueError("sort_alpha参数必须是布尔类型")
    
    if not isinstance(recursive, bool):
        raise ValueError("recursive参数必须是布尔类型")
    
    # 检查目录是否存在
    if not dir_exists(directory):
        return -1
    
    try:
        files = []
        
        # 处理多个文件模式（用 | 分隔）
        patterns = pattern.split("|")
        patterns = [p.strip() for p in patterns]
        
        # 根据是否递归选择不同的遍历方法
        if recursive:
            # 递归遍历所有子目录
            for root, _, filenames in os.walk(directory):
                for filename in filenames:
                    # 检查文件是否匹配任一模式
                    for pat in patterns:
                        if fnmatch.fnmatch(filename, pat):
                            if with_path:
                                files.append(os.path.join(root, filename))
                            else:
                                files.append(filename)
                            break  # 匹配到一个就够了，避免重复添加
        else:
            # 只遍历当前目录
            for filename in os.listdir(directory):
                file_path = os.path.join(directory, filename)
                # 检查是否为文件（而不是目录）
                if os.path.isfile(file_path):
                    # 检查文件是否匹配任一模式
                    for pat in patterns:
                        if fnmatch.fnmatch(filename, pat):
                            if with_path:
                                files.append(file_path)
                            else:
                                files.append(filename)
                            break  # 匹配到一个就够了，避免重复添加
        
        # 如果需要按字母排序
        if sort_alpha:
            files.sort()
        
        return files
    except Exception:
        return -1


def file_size(file_path, unit="M"):
    """
    获取文件大小
    
    Args:
        file_path (str): 文件路径
        unit (str): 返回大小的单位(B、KB、MB、GB)，默认为M
        
    Returns:
        str or int: 成功时返回格式化的文件大小字符串，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    if not isinstance(unit, str):
        raise ValueError("单位必须是字符串类型")
    
    # 检查文件是否存在
    if not file_exists(file_path):
        return -1
    
    try:
        # 获取文件大小（字节）
        size_bytes = os.path.getsize(file_path)
        
        # 定义单位映射
        units = {
            'B': 0,
            'KB': 1,
            'MB': 2,
            'GB': 3
        }
        
        # 转换为大写并检查单位是否有效
        unit = unit.upper()
        if unit not in units:
            unit = "MB"  # 默认使用MB
        
        # 计算对应的单位大小
        if size_bytes == 0:
            return f"0 {unit}"
        
        # 计算大小
        size = size_bytes / (1024 ** units[unit])
        
        # 格式化输出
        if size >= 1000:
            return f"{size:.2f} {unit}"
        elif size >= 100:
            return f"{size:.3f} {unit}"
        elif size >= 10:
            return f"{size:.4f} {unit}"
        else:
            return f"{size:.5f} {unit}"
            
    except Exception:
        return -1


def file_get_extension(file_path):
    """
    获取文件扩展名
    
    Args:
        file_path (str): 文件路径
        
    Returns:
        str or int: 成功时返回文件扩展名（如：.jpg），没有扩展名时返回空字符串，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    try:
        # 使用os.path.splitext获取文件扩展名
        _, extension = os.path.splitext(file_path)
        return extension
    except Exception:
        return -1


def file_get_directory(file_path):
    """
    获取文件所在目录路径
    
    Args:
        file_path (str): 文件路径
        
    Returns:
        str or int: 成功时返回文件所在目录路径，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    try:
        # 使用os.path.dirname获取文件所在目录
        directory = os.path.dirname(file_path)
        
        # 如果目录为空，说明是相对路径的文件，返回当前目录
        if directory == "":
            return "."
        
        # 确保目录路径末尾有路径分隔符
        if not directory.endswith(os.sep):
            directory += os.sep
            
        return directory
    except Exception:
        return -1


def file_get_info(file_path):
    """
    获取文件信息
    
    Args:
        file_path (str): 文件路径
        
    Returns:
        dict or int: 成功时返回包含文件信息的字典，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    # 检查文件是否存在
    if not file_exists(file_path):
        return -1
    
    try:
        # 获取文件统计信息
        stat_info = os.stat(file_path)
        
        # 获取文件基本信息
        file_info = {
            "name": os.path.basename(file_path),  # 文件名
            "path": file_path,  # 完整路径
            "directory": file_get_directory(file_path),  # 所在目录
            "extension": file_get_extension(file_path),  # 扩展名
            "size_bytes": stat_info.st_size,  # 文件大小（字节）
            "size_formatted": file_size(file_path, "MB"),  # 格式化文件大小
            "created_time": time.ctime(stat_info.st_ctime),  # 创建时间
            "modified_time": time.ctime(stat_info.st_mtime),  # 修改时间
            "accessed_time": time.ctime(stat_info.st_atime),  # 访问时间
            "is_readable": os.access(file_path, os.R_OK),  # 是否可读
            "is_writable": os.access(file_path, os.W_OK),  # 是否可写
            "is_executable": os.access(file_path, os.X_OK)  # 是否可执行
        }
        
        return file_info
    except Exception:
        return -1


def file_get_name(file_path, with_extension=False):
    """
    获取文件名
    
    Args:
        file_path (str): 文件路径
        with_extension (bool): 是否带后缀，默认为False，不带后缀；True为带后缀
        
    Returns:
        str or int: 成功时返回文件名，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    if not isinstance(with_extension, bool):
        raise ValueError("with_extension参数必须是布尔类型")
    
    try:
        # 获取文件名（带后缀）
        file_name_with_ext = os.path.basename(file_path)
        
        # 如果需要带后缀，直接返回
        if with_extension:
            return file_name_with_ext
        
        # 如果不需要带后缀，去掉后缀后再返回
        file_name_without_ext = os.path.splitext(file_name_with_ext)[0]
        return file_name_without_ext
    except Exception:
        return -1


def file_delete(file_path, to_trash=True):
    """
    删除文件
    
    Args:
        file_path (str): 文件路径
        to_trash (bool): 是否删除到回收站，默认为True（删除到回收站）；False为彻底删除
        
    Returns:
        bool or int: 成功时返回True，失败时返回-1
    """
    if not isinstance(file_path, str):
        raise ValueError("文件路径必须是字符串类型")
    
    if not isinstance(to_trash, bool):
        raise ValueError("to_trash参数必须是布尔类型")
    
    # 检查文件是否存在
    if not file_exists(file_path):
        return -1
    
    try:
        if to_trash:
            send2trash(file_path)
        else:
            # 彻底删除
            os.remove(file_path)
        
        return True
    except Exception:
        return -1


if __name__ == "__main__":
    # 测试文件存在检查
    print(f"当前文件是否存在: {file_exists(__file__)}")
    print(f"不存在的文件: {file_exists('nonexistent.txt')}")
    print(f"当前目录是否存在: {dir_exists('.')}")
    
    # 测试文件占用检查
    # 使用原始字符串避免转义字符问题
    test_file_path = r'G:\python\yoloProUP\dist\YOLO免环境自动标注训练工具专业版\shortcuts.ini'
    try:
        print(f"当前文件是否被占用: {file_is_use(test_file_path)}")
    except FileNotFoundError:
        print(f"文件不存在: {test_file_path}")
    except Exception as e:
        print(f"检查文件占用状态时发生错误: {e}")
    
    # # 测试文件打开功能
    # print(f"打开当前文件: {file_open(__file__)}")
    # print(f"打开不存在的文件: {file_open('nonexistent.txt')}")
    #
    # # 测试文件执行功能
    # # 注意：这里只是示例，实际使用时请提供存在的文件路径
    # print(f"执行文件: {file_execute(test_file_path)}")
    # print(f"执行不存在的文件: {file_execute('nonexistent.txt')}")
    
    # # 测试文件定位功能
    # print(f"定位当前文件: {file_locate(__file__)}")
    # print(f"定位不存在的文件: {file_locate('nonexistent.txt')}")
    #
    # # 测试文件复制功能
    # print(f"复制当前文件到临时位置: {file_copy(__file__, 'temp_test_copy.py')}")
    # # print(f"复制当前文件到临时位置(覆盖): {file_copy(__file__, 'temp_test_copy.py', True)}")
    # print(f"复制不存在的文件: {file_copy('nonexistent.txt', 'temp_nonexistent_copy.py')}")
    # 
    # # 测试文件重命名功能
    # # 先创建一个测试文件
    # with open('temp_rename_test.txt', 'w') as f:
    #     f.write('test content')
    # print(f"重命名文件: {file_rename('temp_rename_test.txt', 'temp_renamed_test.txt')}")
    # # 清理测试文件
    # if file_exists('temp_renamed_test.txt'):
    #     os.remove('temp_renamed_test.txt')
    # print(f"重命名不存在的文件: {file_rename('nonexistent.txt', 'temp_renamed_test.txt')}")
    # 
    # # 测试文件枚举功能
    # print(f"枚举当前目录所有文件: {file_enumerate('.', '*.py',recursive=False)}")
    # print(f"枚举当前目录txt文件: {file_enumerate('.', '*.py')}")
    # print(f"枚举当前目录py和txt文件: {file_enumerate('.', '*.py|*.txt')}")
    # print(f"枚举当前目录所有文件(带路径): {file_enumerate('.', '*.*', with_path=True)}")
    # print(f"枚举当前目录所有文件(排序): {file_enumerate('.', '*.*', sort_alpha=True)}")
    # print(f"枚举当前目录所有文件(带路径+排序): {file_enumerate('.', '*.*', with_path=True, sort_alpha=True)}")
    # 
    # 测试文件大小功能
    # print(f"当前文件大小(MB): {file_size(__file__)}")
    # print(f"当前文件大小(B): {file_size(__file__, 'B')}")
    # print(f"当前文件大小(KB): {file_size(__file__, 'KB')}")
    # print(f"当前文件大小(GB): {file_size(__file__, 'GB')}")
    # print(f"不存在文件的大小: {file_size('nonexistent.txt')}")
    # 
    # 测试文件扩展名功能
    # print(f"当前文件的扩展名: {file_get_extension(__file__)}")
    # print(f"没有扩展名的文件: {file_get_extension('README')}")
    # print(f"带扩展名的文件: {file_get_extension('test.txt')}")
    # print(f"多级扩展名: {file_get_extension('archive.tar.gz')}")
    # 
    # 测试文件目录功能
    # print(f"当前文件所在目录: {file_get_directory(__file__)}")
    # print(f"绝对路径文件所在目录: {file_get_directory(r'C:\\012\\3600.exe')}")
    # print(f"相对路径文件所在目录: {file_get_directory('folder/test.txt')}")
    # 
    # 测试文件信息功能
    # info = file_get_info(__file__)
    # if info != -1:
    #     print("当前文件信息:")
    #     for key, value in info.items():
    #         print(f"  {key}: {value}")
    # else:
    #     print("获取文件信息失败")
    # 
    # print(f"不存在文件的信息: {file_get_info('nonexistent.txt')}")
    # 
    # 测试文件名功能
    # print(f"当前文件名(不带后缀): {file_get_name(__file__)}")
    # print(f"当前文件名(带后缀): {file_get_name(__file__, True)}")
    # print(f"C:\\123.exe(不带后缀): {file_get_name(r'C:\\123.exe')}")
    # print(f"C:\\123.exe(带后缀): {file_get_name(r'C:\\123.exe', True)}")
    # print(f"不存在文件的文件名: {file_get_name('nonexistent.txt')}")
    # 
    # 测试文件删除功能
    # # 先创建一个测试文件
    # with open('temp_delete_test.txt', 'w') as f:
    #     f.write('test content for deletion')
    # print(f"删除文件到回收站: {file_delete('temp_delete_test.txt')}")
    # 
    # # 再创建一个测试文件
    # with open('temp_delete_test2.txt', 'w') as f:
    #     f.write('test content for permanent deletion')
    # print(f"彻底删除文件: {file_delete('temp_delete_test2.txt', False)}")
    # 
    # print(f"删除不存在的文件: {file_delete('nonexistent.txt')}")